/* arg_list_api.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

/*
 * arg_list_api.h
 *
 * Functions to work with a list of arguments
 *
 */
#ifndef ARG_LIST_API_H
#define ARG_LIST_API_H

/******************************************************************************
 *				Include Files
 *****************************************************************************/

#include <stdio.h>

/******************************************************************************
 *				Macro Definitions
 *****************************************************************************/

#define AL_TYPE_UNDEF		0
#define AL_TYPE_INT		1	/* Argument contains an int      */
#define AL_TYPE_STRING		2	/* Argument contains a char *    */
#define AL_TYPE_VOID		3	/* Argument contains a void *    */
#define AL_TYPE_LIST		4	/* Argument contains ARG_LIST    */
#define AL_TYPE_HND		5	/* Argument contains ARG_HND    */
		/* Macro definitions used for type arguments */

#define AL_E_OK			0	/* OK */
#define AL_E_WRONG_TYPE		1	/* Wrong type of argument requested */
#define AL_E_NOT_FOUND		2	/* Argument not found */
#define AL_E_END_OF_LIST	3	/* Last element in a list encountered */
#define AL_E_DUP		4	/* argument already exists      */
#define AL_E_NOT_IMPL		5	/* Not implemented */

		/* Return values from find and add routines */

/******************************************************************************
 *				Type Definitions
 *****************************************************************************/

typedef struct arg_handle_str *ARG_HND;
		/*  A handle to a data structure that can contain a 
		   collection of named arguments.
		 */

typedef struct arg_list_str *ARG_LIST;
		/*  A collection of ARG_ENTRY structures */

typedef void *ARG_ENTRY;
		/*  One element of an ARG_LIST           */

/******************************************************************************
 *				Function Prototypes
 *****************************************************************************/

ARG_HND al_hnd_new (void);
void al_hnd_delete (ARG_HND * hnd_ptr);
int al_hnd_add_string (ARG_HND hnd, const char *name, const char *value);
int al_hnd_add_int (ARG_HND hnd, char *name, int value);
int al_hnd_add_void (ARG_HND hnd, char *name, const void *value);
int al_hnd_add_list (ARG_HND hnd, char *name, ARG_LIST list);
int al_hnd_add_hnd (ARG_HND hnd, char *name, ARG_HND value);

int al_hnd_find (ARG_HND hnd, char *name, int *type_ptr, void **value_ptr);
char *al_hnd_find_string (ARG_HND hnd, char *name);
int al_hnd_find_int (ARG_HND hnd, char *name);
void *al_hnd_find_void (ARG_HND hnd, char *name);
ARG_LIST al_hnd_find_list (ARG_HND hnd, char *name);
ARG_HND al_hnd_find_hnd (ARG_HND hnd, char *name);

ARG_LIST al_hnd_get_names (ARG_HND hnd);
int al_hnd_declare_type (ARG_HND hnd, char *name);
ARG_HND al_hnd_read (FILE * in_file, char *name, int namelen);
ARG_HND al_hnd_read_string (char *in_string, char *name, int namelen);
int al_hnd_write (ARG_HND hnd, FILE * out_file, char *name);

ARG_LIST al_list_new (void);
void al_list_delete (ARG_LIST * list_ptr);
int al_list_add_string (ARG_LIST list_ptr, const char *value);
int al_list_length (ARG_LIST list);
ARG_ENTRY al_list_first (ARG_LIST list);
ARG_ENTRY al_list_next (ARG_ENTRY list);
int al_list_string_exists (ARG_LIST list, char *value);

char *al_entry_get_string (ARG_ENTRY entry);

#endif /* ARG_LIST_API_H */
