/* db.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

/*
 * db.h
 *
 *  To make things fast (and most important easy :) alot of the structures, etc
 * are created around the fact these routines will be used for dents.
 *
 * There are 3 main uses for these type of trees:
 *
 *  1) Storing of RR's for cache, authoritative information and hints
 *  2) List of zones we are authoritative for
 *  3) Outstanding queries
 */
#ifndef DB_H
#define DB_H

#include <sys/types.h>
#include "db_rb.h"

#define DB_MAXLABELLENGTH 64
#define DB_MAXNAMELENGTH 255

typedef struct
{
  /* Size of hash table (# of bits) */
  guint8 hashsize;

  /* Number of entries we have in this level of the db */
  guint32 numentries;

  /* Pointer to array for hash index */
  void *table;
}
dbhashrec;

typedef struct
{
  void *parent;			/* Pointer to parent node */
  dbrbnode rb;			/* Red-Black tree node */

  guchar label[DB_MAXLABELLENGTH + 1];	/* Label for this node */

  dbrb children;		/* Red-Black tree for the children */

  void *data;			/* Pointer to the data for this node */
}
dbnoderec;

typedef struct
{
  /* Number of entries we have in this db */
  guint32 numentries;

  /* Pointer to this level */
  dbnoderec node;
}
dbrootrec;

/* Prototypes */

/* db.c */
void db_deletenode (dbnoderec * node);
dbnoderec *db_finddname (dbrootrec * db, guchar * domainname);
void db_init (dbrootrec * db);
dbnoderec *db_addnode (dbnoderec * node, guchar * label);
dbnoderec *db_findnode (dbnoderec * node, guchar * label);
#ifdef DENTS_DEBUG
void db_display (dbrootrec * db);
#endif

#endif
