/* dns_types.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

/*
 * dns_types.h
 *
 * Basically I was tired of seeing all of the RR type's etc cluttering
 * my beautiful dns.h file so I moved them here -jerdfelt
 *
 * To add an RR, follow these steps:
 * 1) Add a #define for DT_XXX 
 * 2) Add a member to dnsrrtyperec[] in dns_types.c
 * 3) Add a case statement to dns_types_pack() and dns_types_unpack() 
 *      in dns_types.c
 * 4) Add a case statement to dns_compress_rr() in dns_comp.c
 * 5) Add a case statement to dns_dt_to_string() in dns_debug.c
 * 6) Add a case statement to _debug_print_pkt_rr() in dns_debug.c
 *
 * FIXME: Verify the steps
 *
 * A arg count of -1 means the RR is obsolete and generate a warning
 */
#ifndef DNS_TYPES_H
#define DNS_TYPES_H

#include "db.h"

#include <netinet/in.h>

/* type structure containing some info about it and node for RB tree */
typedef struct
{
  gint8 numargs;		/* Number of arguments (or -1 for obsolete) */

  guint8 qtype;			/* qtype num (per RFC) */

  dbrbnode labelnode;		/* For RB tree to speed up lookups */
  char *label;			/* ASCII string used for text zone files */
}
dnsrrtyperec;

/* Allowed QType's */
#define DT_A		1	/* IPv4 Address */
#define DT_NS		2	/* Authoritative Server */
#define DT_MD		3	/* Mail Destination - Obsolete */
#define DT_MF		4	/* Mail Forwarder - Obsolete */
#define DT_CNAME	5	/* Canonical Name */
#define DT_SOA		6	/* Start Of Authority */
#define DT_MB		7	/* Mailbox Domain Name - Experimental */
#define DT_MG		8	/* Mail Group Member - Experimental */
#define DT_MR		9	/* Mail Rename Name - Experimental */
#define DT_NULL		10	/* null RR */
#define DT_WKS		11	/* Well Known Service */
#define DT_PTR		12	/* Pointer */
#define DT_HINFO	13	/* Host Information */
#define DT_MINFO	14	/* Mailbox Information */
#define DT_MX		15	/* Mail Exchanger */
#define DT_TXT		16	/* Text */
#define DT_RP		17	/* Responsible Person */
#define DT_AFSDB	18	/* AFS Cell Database */
#define DT_X25		19	/* X_25 Calling Address */
#define DT_ISDN		20	/* ISDN Calling Address */
#define DT_RT		21	/* Router */
#define DT_NSAP		22	/* NSAP Address */
#define DT_NSAP_PTR	23	/* Reverse NSAP lookup - deprecated */
#define DT_SIG		24	/* Security Signature */
#define DT_KEY		25	/* Security Key */
#define DT_PX		26	/* X.400 Mail Mapping */
#define DT_GPOS		27	/* Geographical Position - withdrawn */
#define DT_AAAA		28	/* IPv6 Address */
#define DT_LOC		29	/* Location Information */
#define DT_NXT		30	/* Next Valid Name in Zone */
#define DT_EID		31	/* Endpoint Identifier */
#define DT_NIMLOC	32	/* Nimrod Locator */
#define DT_SRV		33	/* Server Selection */
#define DT_ATMA		34	/* ATM Address */
#define DT_NAPTR	35	/* Naming Authority Pointer */
/* These next 4 QType's are Non Standard */
#define DT_UINFO	100	/* User Information */
#define DT_UID		101	/* User ID */
#define DT_GID		102	/* Group ID */
#define DT_UNSPEC	103	/* Unspecified (Binary Data) */
/* These next 5 QType's can't appear as RR's */
#define DT_IXFR		251	/* Incremental Zone Transfer */
#define DT_AXFR		252	/* Transfer Zone Of Authority */
#define DT_MAILB	253	/* Transfer Mailbox Records */
#define DT_MAILA	254	/* Transfer Mail Agent Records */
#define DT_ANY		255	/* Wildcard Match */

/* Allowed QClass' */
#define DC_IN		1	/* Internet */
#define DC_CS		2	/* CSNET - Obsolete */
#define DC_CH		3	/* Chaos */
#define DC_HS		4	/* Hesiod */
/* This can't appear as an RR, only in a query */
#define DC_ANY		255	/* Any Class */

/* DNS Opcodes */
#define DO_STANDARD	0
#define DO_INVERSE	1
#define DO_STATUS	2

/* DNS Error Codes */
#define DE_WAITFORMOREINFO	-2
#define DE_NOINFO	-1
#define DE_NOERROR	0
#define DE_FORMERR	1
#define DE_SERVFAIL	2
#define DE_NXDOMAIN	3
#define DE_NOTIMP	4
#define DE_REFUSED	5

#endif
