/* mod_mysql_eza.c - DENTS module that uses a MySQL database to store data
     Copyright (C) 1999  Eric Z. Ayers  (Eric.Ayers@mindspring.com)

   Dents - a threaded modular dns-server.
     Copyright (C) 1999  Dents Team  (http://www.dents.org)
  

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  

   See DESCRIPTION for more information about using this module.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/*
 * $Log: mod_mysql_eza.c,v $
 * Revision 1.14  1999/09/25 14:49:56  illume
 * I removed "#ifdef WITH_MYSQL".  It reminds me of a DOS-program that
 * told the user whether the computer was switched on or not. :-)
 *
 * Revision 1.13  1999/08/08 03:56:02  eric
 * no, really... THIS is the last change I plan to make for a while (didn't compile)
 *
 * Revision 1.12  1999/08/08 03:37:56  eric
 *
 * Added the SOA and Authority record cache.  I am done working on this
 * module for a while, unless people report bugs.
 *
 * Revision 1.11  1999/07/30 20:58:02  illume
 * Several changes made while trying to compile dents on an old solaris
 * host.
 *
 * Revision 1.10  1999/07/28 11:45:02  eric
 * pulled out logic around mysql_query() into a separate function call.
 *
 * Revision 1.9  1999/07/27 11:22:28  eric
 * Added the ability to reconnect to the database.
 *
 * Revision 1.8  1999/07/11 12:22:29  eric
 * added debugging message as an example of how you can tell where a packet
 * originated from.
 *
 * Revision 1.7  1999/07/10 16:38:43  eric
 * change errors for loading zone to err_warn() instead of DPRINTF
 *
 * Revision 1.6  1999/07/05 18:48:09  eric
 * took care of compiler warnings with -Wall.
 *
 * Revision 1.5  1999/07/04 17:07:31  eric
 * updated to use dns_types_pack_XXX routines and handle DT_ANY queries.
 * the module *should* be thread safe now.
 *
 * Revision 1.4  1999/07/03 17:39:23  jerdfelt
 * Merged in Paul Kunysch's auto* fixes
 *
 * Revision 1.3  1999/06/06 01:02:26  eric
 * added copyright notice, MX records, MINFO and HINFO support
 *
 * Revision 1.2  1999/06/05 04:30:46  eric
 * fixed the CNAME return values.
 *
 * Revision 1.1  1999/06/05 02:45:22  eric
 *
 *
 * These files were moved from mod_mysql to mod_mysql_eza
 *
 * Revision 1.1  1999/06/05 02:32:05  eric
 *
 *
 * just renamed mod_mysql.c
 *
 * Revision 1.2  1999/06/02 20:45:45  eric
 * Added test for MYSQL in configure.in - now there is a mod_mysql_eza_config.h
 * file
 *
 * Revision 1.1  1999/05/13 03:03:12  eric
 *
 * configure template
 *
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/***************************************************************************
 *			Module Decls Definitions
 ***************************************************************************/

/* Basic info on the module, used later */
/* the following macros must be defined before including moudle_syms.h */

#define MODULE_NAME		mod_mysql_eza
#define MODULE_NAME_STRING	"mod_mysql_eza"

#define CACHE_EXPIRATION_SECS	3600
	/* # of seconds the cached SOA and NS records are valid */

#define MODULE_VER_MAJOR 1
#define MODULE_VER_MINOR 0
#define MODULE_VER_MICRO 1

/***************************************************************************
 *			Include Files
 ***************************************************************************/
/* UNIX includes */
#include <signal.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <assert.h>

/* DENTS includes */
#include "dns.h"
#include "module_syms.h"
#include "dns_module_syms.h"
#include "link.h"
#include "addr_tools.h"

/* MySQL includes */
#include <mysql/mysql.h>

/***************************************************************************
 *			Macro Definitions
 ***************************************************************************/

#define  MMH_MAGIC 			0x34EA56FF
		/* Randomly chosen identifier to detect memory probs */
#define  MMC_MAGIC 			0x36EA46EE
		/* Randomly chosen identifier to detect memory probs */

#define CHECK_MMH(ptr) \
{ ASSERT(ptr); ASSERT(ptr->mmh_magic == MMH_MAGIC); }

#define CHECK_MMC(ptr) \
{ ASSERT(ptr); ASSERT(ptr->mmc_magic == MMC_MAGIC); }


#define QUERYLEN	256
	/* Size of buffer to declare for composing queries.  bounds are
	   checked with snprintf()
	*/

#if ASSERT_IS_BROKEN
#define ASSERT(arg) \
if (0 == (long)(arg)) \
{ \
	int mypid = getpid(); \
	(void)fprintf(stderr, "Assertion Failed in %s at line %d\n", \
	       __FILE__, __LINE__ ); \
	(void)fflush(stderr); \
	(void)kill(mypid, SIGABRT); \
}
				/* a macro to halt execution and force  */
				/* a core dump for debugging analysis   */
#else
#define ASSERT(arg)  assert(arg)
#endif

#define NOT_IMPL(arg) DPRINTF(("%s: NOT IMPLEMENTED %s", ROUTINE, (arg)))

/***************************************************************************
 *			Type Definitions
 ***************************************************************************/
/*
 * MOD_MYSQL_HND 
 *
 * This structure is a handle to an open database.  Several zones may
 * share the same database handle. The key to this structure is 
 *  hostname, db_name, and user_name.
 *
 */
typedef struct mod_mysql_hnd_str
{
  unsigned int mmh_magic;
  /* Guard bits to catch memory corruption          */

  llentry mmh_ll;
  /* Linked list data structure.                    */

  char *mmh_host_name;
  /* host to connect to                             */
  char *mmh_db_name;
  /* database name to connect to                    */
  char *mmh_user_name;
  /* username to login to the database              */
  char *mmh_password;
  /* password for this user                         */

  MYSQL mmh_mysql_hnd;
  /* MYSQL database handle                          */

  int mmh_mysql_hnd_init;
  /* set to 1 if the handle above is valid.  Set    */
  /* to zero otherwise                              */

  int	mmh_ref_cnt;
  /* # of times this handle is referenced by a zone */
				   
} MOD_MYSQL_HND;


typedef struct mod_mysql_cache_str {
  unsigned int 	mmc_magic;
  /* Guard bits to catch memory corruption	  */
  
  char mmc_domain_name[DNS_MAXNAMELENGTH];
  /* Name of the domain this entry describes	  */

  MOD_MYSQL_HND *mmc_hnd;
  /* Database handle used by this zone		  */

  time_t	mmc_cache_time;
  /* time this cache was built (so it can be refreshed) */

  char mmc_soa_mname[DNS_MAXNAMELENGTH];
  char mmc_soa_rname[DNS_MAXNAMELENGTH];
  u_int32_t mmc_soa_serial;
  int32_t mmc_soa_refresh;
  int32_t mmc_soa_retry;
  int32_t mmc_soa_expire;
  u_int32_t mmc_soa_minimum;

  guchar *mmc_soa_rdata;
  int mmc_soa_rdata_len;
  /* Cached SOA information */

  GSList	*mmc_auth_rrs;
  /* Cached NS Records (Authority Info) (a list of MOD_MYSQL_RRs)*/
  
  GSList	*mmc_addit_rrs;
  /* Cached A Records for above NS Records (Additional Info) 
     (list of MOD_MYSQL_RRs) */

} MOD_MYSQL_CACHE;


/*
 * MOD_MYSQL_RR - Structure for storing RR's in the cache handle.
 *
 * Why didn't I reuse dnsrrrec or dnsdbrrrec in dns.h?  because
 * those structures are wired into a tree/list structure, OR they
 * are missing the rdata field.  If I passed one of those to 
 * a DENTS function, it would assume that this was linked up 
 * with a tree of RRs which is is not.
 */
typedef struct mod_mysql_rr_str
{
  guint16 mmr_type;		/* Type of RR */
  guint16 mmr_class;		/* Class of RR */
  gint32  mmr_ttl;		/* TTL of RR */
  int     mmr_rdata_length;	/* Length of the RDATA */
  guchar  *mmr_rdata;		/* The RDATA (allocated to size) */

} MOD_MYSQL_RR;


/***************************************************************************
 *			Static Module Data
 ***************************************************************************/

static linkedlist gbl_hnd_list = LINK_INITIALIZER (MOD_MYSQL_HND, mmh_ll);
		/* List of MOD_MYSQL_HND                          
		   Open handles to MYSQL databases. This module will
		   re-use handles between zones and then closes them
		   when the last zone referencing a handle is unloaded.

		   NOTE: watch out for the database closing a handle
		   that hasn't been used in a while!
		*/


/***************************************************************************
 *			Function Prototypes
 ***************************************************************************/


static MOD_MYSQL_HND * find_mmh 
(
 char * host_name, 
 char * db_name, 
 char * user_name
 );

static void del_mmh 
(
 MOD_MYSQL_HND *hnd 
 );

static char * make_table_name 
(
 char * zone_name, 
 char *table_name
);

static int connect_to_db 
(
 MOD_MYSQL_HND	*hnd,
 char		*zone_name
);

static int query_db
(
 char 		*query,
 MOD_MYSQL_HND  *hnd, 
 char 		*zone_name
);

static MOD_MYSQL_CACHE * cache_auth_and_addit
(
 MOD_MYSQL_HND 	*hnd, 
 char		*zone_name
);

/***************************************************************************
 ***************************************************************************
 **			Internally Visible Functions
 ***************************************************************************
 ***************************************************************************/

/*
 * find_mmh 
 *
 * Find the database handle in the gbl_hnd_list that
 * matches all three of host_name, db_name and user_name.
 * Return NULL if not found.
 */
static MOD_MYSQL_HND *find_mmh 
(char *host_name, char *db_name, char *user_name)
{
#define ROUTINE "find_mmh ()"
  MOD_MYSQL_HND *entry;

  DPRINTF (("%s: Starting: %s %s %s\n",
	    ROUTINE, host_name, db_name, user_name));

  for (entry = link_first (&gbl_hnd_list);
       entry; entry = link_next (entry, entry->mmh_ll)) {

    if (!strcmp (entry->mmh_host_name, host_name)
	&& !strcmp (entry->mmh_db_name, db_name)
	&& !strcmp (entry->mmh_user_name, user_name)) {
      return entry;
    }
  }				/* end for each entry in the list */

  return NULL;

#undef ROUTINE
}				/* end find_mmh() */



/*
 * del_mmh
 *
 * Delete a handle from the list and free memory IF the reference
 * count has decremented to 0.
 */
static void
del_mmh (MOD_MYSQL_HND * hnd)
{
#define ROUTINE "del_mmh  ()"
  DPRINTF (("%s: Starting: %s %s %s\n",
	    ROUTINE, hnd->mmh_host_name, hnd->mmh_db_name,
	    hnd->mmh_user_name));

  CHECK_MMH (hnd);

  hnd->mmh_ref_cnt--;

  if (hnd->mmh_ref_cnt <= 0) {

    /* Disconnect from the database */

    /* remove the entry from the list of all open handles */
    link_delete (&gbl_hnd_list, &hnd->mmh_ll);

    /* Free other memory associated with the handle */
    g_free (hnd->mmh_host_name);
    g_free (hnd->mmh_db_name);
    g_free (hnd->mmh_user_name);
    g_free (hnd->mmh_password);

    /* munge the handle up */
    memset (hnd, 0, sizeof (*hnd));

    g_free (hnd);
  }
#undef ROUTINE
}				/* end del_mmh () */

/*
 * make_table_name
 * Given a DNS zone name, construct the name of the MYSQL table that
 * holds the Resource Records for it.
 *
 * Inputs:
 *   zone_name - name of DNS zone to be loaded
 * Outputs:
 *   table_name - name of mysql table
 *
 * Returns the second argument for convenience if you want to use
 * this function inline with another function call.
 *
 */
static char *
make_table_name (char *zone_name, char *table_name)
{
  char *p, *q;			/* scratch pointers      */

  /* Translate periods to underscores for the table name. The underscore
     character is illegal in the DNS protocol and periods are illegal 
     in mysql table names.
   */
  strcpy (table_name, "zone_rr_");
  q = &table_name[strlen (table_name)];

  for (p = zone_name; *p; p++, q++) {
    if (*p == '.')
      *q = '_';
    else
      *q = *p;
  }
  *q = 0;			/* null terminate table name */

  return table_name;
}				/* end make_table_name () */

/*
 * connect_to_db - connect or reconnect to the MYSQL server
 *
 * This performs the connect and specifies the database.
 *
 * Inputs: hnd - A populated MOD_MYSQL handle. This should have the db name,
 *               hostname, username and password already populated.
 *         zone_name - a valid DNS zone name (from a dnszonerec ptr)
 *
 * Returns: non-zero on failure, zero on success.
 *	
 * If sucessful, the field hnd->mmh_mysql_hnd_init will be 1
 *
 */

static int connect_to_db ( MOD_MYSQL_HND *hnd, char *zone_name)
{
#define ROUTINE "connect_to_db ()"
  char table_name[DNS_MAXNAMELENGTH + 1];	/* name of table  in db */
  MYSQL_RES *mresult;	/* mysql structure for holding a query result */

  CHECK_MMH (hnd);

  if (hnd->mmh_mysql_hnd_init)
    return 0;

  /* Connect to the database, (login to MYSQL server) */
  if (!mysql_connect(&hnd->mmh_mysql_hnd, hnd->mmh_host_name, 
		     hnd->mmh_user_name, hnd->mmh_password)) {
    err_warn("%s: %s; zone %s MYSQL login failed! Error %s (%d)\n", 
	     MODULE_NAME_STRING, ROUTINE, zone_name,
	     mysql_error(&hnd->mmh_mysql_hnd),
	     mysql_errno(&hnd->mmh_mysql_hnd));

    return -1;
  } 


  /* Open the specified database */
  if (mysql_select_db (&hnd->mmh_mysql_hnd, hnd->mmh_db_name)) {
    err_warn ("%s: %s; zone %s MYSQL select DB %s failed! %s (%d)\n", 
	      MODULE_NAME_STRING, ROUTINE, zone_name,
	      hnd->mmh_db_name,
	      mysql_error(&hnd->mmh_mysql_hnd),
	      mysql_errno(&hnd->mmh_mysql_hnd));

    return -1;
  }

  /* look for a table named 'res_<zone_name>  in the database */
  make_table_name (zone_name, table_name);

  if (!(mresult = mysql_list_tables (&hnd->mmh_mysql_hnd, table_name))) {

    err_warn ("%s: mysql_list_tables(%s): %s (%d)\n",
	      ROUTINE,
	      table_name,
	      mysql_error (&hnd->mmh_mysql_hnd),
	      mysql_errno (&hnd->mmh_mysql_hnd));
    return -1;
  }

  /* Check to see if any tabgle exists with this name */
  if (mysql_num_rows (mresult) <= 0) {
    err_warn ("%s: no rows returned from query for %s\n", ROUTINE,
	      table_name);
    mysql_free_result (mresult);

    return -1;
  }

  return 0;

#undef ROUTINE
}				/* end connect_to_db() */



/* 
 * query_db - send the query to the server using msql_query.
 *
 * Description: 
 *      This function wraps msql_query() and does an automatic
 *      attempt to reconnect if the query fails.
 *
 *      Try this query up to two times.  If it fails the first time,
 *      try to reconnect to the deamon.  Note that the query will not
 *      fail if there are simply no results returned.  I think that
 *      the query could fail for a lot of reasons that re-connecting could
 *      help with.  I don't see a specific errro in mysql/errmsg.h that
 *      indicates a parse error, but that shouldn't make it into 
 *      production!
 *
 * Inputs:      
 *      query - the SQL query to execute
 *      hnd - a valid MOD_MYSQL_HND
 *      zone - pointer to a dnszonerec passed to the answer() function 
 *              by DENTS.
 * Outputs:
 *      None.
 * Returns: 
 *      non-zero on failure
 *      zero on success - the code must call mysql_store_result() to get data
 */
static int query_db ( char * query, MOD_MYSQL_HND *hnd, char *zone_name )
{
#define ROUTINE "query_db ()"
  int i;

  for (i = 0; i < 2; i++) {

    /* NOTE: it is possible the server connection will time out.
       this code tries to be smart and reconnect and retry at this 
       point...
       
       A previous call could have caused the connection to be closed,
       If it does, the mmh_mysql_hnd_init will be 0. */

    if (hnd->mmh_mysql_hnd_init
	&& (mysql_query (&hnd->mmh_mysql_hnd, query) == 0)) {

      /* the query succeeded, break out of the loop */
      return 0;

    } else {
      DPRINTF (("query failed: %s (%d)\n", mysql_error (&hnd->mmh_mysql_hnd),
		mysql_errno (&hnd->mmh_mysql_hnd)));
      /* attempt to reconnect and then retry the query */
    }

    err_warn("%s: %s; zone %s: Attempting to reconnect to database %s on %s\n",
	      MODULE_NAME_STRING, ROUTINE, zone_name, 
	     hnd->mmh_db_name, hnd->mmh_host_name);

    /* Attempt to re-connect to the database */
    mysql_close (&hnd->mmh_mysql_hnd);
    hnd->mmh_mysql_hnd_init = 0;

    if (connect_to_db(hnd, zone_name) != 0) {
      /* DON'T delete the handle here... the server may come back! */
      return (-1);
    }
  }				/* end for */

  return 1;
#undef ROUTINE
} /* end query_db */


/*
 * del_mmc - free all memory associated with a MOD_MYSQL_CACHE data structure
 *
 * Inputs:
 *	mmc_ptr - returned from cache_auth_and_addit.  Don't use this pointer
 *        after you call this function!
 * Outputs:
 *	None.
 * Returns:
 * 	None.
 */
void del_mmc (MOD_MYSQL_CACHE *mmc_ptr)
{
#define ROUTINE "del_mmc ()"
  GSList *curr_entry;

  CHECK_MMC(mmc_ptr);

  g_free (mmc_ptr->mmc_soa_rdata);
  
  /* free the list of authority RRs */
  for (curr_entry = mmc_ptr->mmc_auth_rrs;
       curr_entry; 
       curr_entry = curr_entry->next)  {
    MOD_MYSQL_RR * rr_ptr = (MOD_MYSQL_RR *)curr_entry->data;
    g_free(rr_ptr->mmr_rdata);
    g_free(rr_ptr);
  }
  g_slist_free (mmc_ptr->mmc_auth_rrs);
  mmc_ptr->mmc_auth_rrs = NULL;

  /* free the list of Addit RRs */
  for (curr_entry = mmc_ptr->mmc_addit_rrs;
       curr_entry; 
       curr_entry = curr_entry->next)  {
    MOD_MYSQL_RR * rr_ptr = (MOD_MYSQL_RR *)curr_entry->data;
    g_free(rr_ptr->mmr_rdata);
    g_free(rr_ptr);    
  }
  g_slist_free (mmc_ptr->mmc_addit_rrs);
  mmc_ptr->mmc_addit_rrs = NULL;
  
  memset (mmc_ptr, 0, sizeof(*mmc_ptr));

  g_free (mmc_ptr);

#undef ROUTINE
} /* end del_mmc () */


/*
 * cache_auth_and_addit - Remember auth and additional RR info for a zone
 *
 * Since the additional and auth information is returned with every
 * query, it seems like a big performance drain to re-query these values
 * each time.  That'd be at least 3 SQL queries instead of just 1 for
 * every query.  
 *
 * The ultimate goal of dents is to provide an interface for adding RR's,
 * so when RRs are added, this module can automatically update the 
 * serial number field, and will refresh the cached information.
 * 
 * For now, the cached info will expire on a timeout (The TTL or 1 hour, 
 * whichever is less) and will be re-queried on the next lookup for that 
 * domain.
 *
 * Inputs:
 *	hnd - A valid MOD_MYSQL_HND value
 *	zone - a valid zone record passed to a query or a load
 * Outputs:
 *      none
 * Returns:  non-zero on failure, zero on success.
 * 
 */
static MOD_MYSQL_CACHE * cache_auth_and_addit
(
 MOD_MYSQL_HND 	*hnd, 
 char		*zone_name
)
{
#define ROUTINE "cache_auth_and_addit ()"
  char 
    table_name[DNS_MAXNAMELENGTH + 1],       /* name of table  in db */
    query [QUERYLEN+(2*DNS_MAXNAMELENGTH)+1]; /* SQL query to send   */
  MYSQL_RES 
    *mresult;		/* handle for status returned from query */
  MYSQL_ROW mrow;    	/* handle for data returned from query */
  MOD_MYSQL_CACHE
    *mmc_ptr = NULL;	/* New cache buffer to create		*/


  /* Grab the SOA info for this zone from the SOA table */ 
  g_snprintf (query, sizeof(query), 
	      "select mname, rname, serial, refresh, retry, expire, minimum "
	      "from zone_soa where domain='%s'",
	      zone_name);

  if (query_db (query, hnd, zone_name) != 0) {
    err_warn ("%s: %s; zone %s query '%s' for SOA failed: %s (%d)\n", 
	      MODULE_NAME_STRING, ROUTINE, zone_name,
	      query,
	      mysql_error(&hnd->mmh_mysql_hnd),
	      mysql_errno(&hnd->mmh_mysql_hnd));
    return NULL;
  } else if ((mresult = mysql_store_result(&hnd->mmh_mysql_hnd)) == 0) {
    err_warn ("%s: %s; zone %s store result: %s (%d)\n", 
	      MODULE_NAME_STRING, ROUTINE, zone_name,
	      mysql_error(&hnd->mmh_mysql_hnd),
	      mysql_errno(&hnd->mmh_mysql_hnd));
    return NULL;
  } else if (mysql_num_rows(mresult) <= 0) {
    /* need err_warn */
    err_warn ("%s: %s; zone %s No SOA record found\n", 
	       MODULE_NAME_STRING, ROUTINE, zone_name);
    return NULL;
  } else if ((mrow = mysql_fetch_row(mresult)) == NULL) {
    err_warn ("%s: %s; zone %s Couldn't fetch row for SOA record %s (%d)\n", 
	      MODULE_NAME_STRING, ROUTINE, zone_name, 
	      mysql_error(&hnd->mmh_mysql_hnd),
	      mysql_errno(&hnd->mmh_mysql_hnd));
    mysql_free_result(mresult);
    return NULL;
  } else {
    mmc_ptr = (MOD_MYSQL_CACHE *)g_malloc0(sizeof(MOD_MYSQL_CACHE));
    mmc_ptr->mmc_magic = MMC_MAGIC;
    mmc_ptr->mmc_hnd = hnd;
    mmc_ptr->mmc_cache_time = time(NULL);
    strncpyzt (mmc_ptr->mmc_domain_name, zone_name, 
	       sizeof(mmc_ptr->mmc_domain_name));

    /* At this point, we should have results in mrow[0] .. mrow[6]. */
    strncpyzt(mmc_ptr->mmc_soa_mname, mrow[0], sizeof(mmc_ptr->mmc_soa_mname));
    strncpyzt(mmc_ptr->mmc_soa_rname, mrow[1], sizeof(mmc_ptr->mmc_soa_rname));

    /* Don't worry too much about these not being numbers - the
       SQL table defines these as numeric types.
    */
    sscanf (mrow[2], "%u", &mmc_ptr->mmc_soa_serial);
    sscanf (mrow[3], "%d", &mmc_ptr->mmc_soa_refresh);
    sscanf (mrow[4], "%d", &mmc_ptr->mmc_soa_retry);
    sscanf (mrow[5], "%d", &mmc_ptr->mmc_soa_expire);
    sscanf (mrow[6], "%u", &mmc_ptr->mmc_soa_minimum);

    /* build some RDATA */
    mmc_ptr->mmc_soa_rdata_len = 
      dns_types_pack_SOA( NULL, &mmc_ptr->mmc_soa_rdata, zone_name,
			  mmc_ptr->mmc_soa_mname, mmc_ptr->mmc_soa_rname, 
			  mmc_ptr->mmc_soa_serial, mmc_ptr->mmc_soa_refresh,
			  mmc_ptr->mmc_soa_retry, mmc_ptr->mmc_soa_expire,
			  mmc_ptr->mmc_soa_minimum);

    mysql_free_result(mresult);
    DPRINTF(("%s: Got SOA!\n", ROUTINE));
  }
    
  make_table_name (zone_name, table_name);

  /* Lookup NS records in this zone.  These are cached so that we
     can return them with every query.
   */
  query[0]=0;
  g_snprintf (query, sizeof(query), 
	      "Select type, ttl, class, "
	      " rdata_char, rdata_blob\n"
	      " from %s \n"
	      " where domain='' and type=%d ",
	      table_name, DT_NS);

  if (query_db(query, hnd, zone_name) != 0){
    DPRINTF ((ROUTINE ": query for DT_NS failed\n"));
  } else if ((mresult = mysql_store_result(&hnd->mmh_mysql_hnd)) == 0) {
    DPRINTF(("store result for DT_NS: %s (%d)\n", 
	     mysql_error(&hnd->mmh_mysql_hnd),
	     mysql_errno(&hnd->mmh_mysql_hnd)));
  } else if (mysql_num_rows(mresult) <= 0) {
    /* need err_warn */
    err_warn ("%s: %s; zone %s No NS records found\n", 
	       MODULE_NAME_STRING, ROUTINE, zone_name);
    DPRINTF ((ROUTINE ": TODO - set some defaults for missing SOA(?!?)\n"));
  } else if ((mrow = mysql_fetch_row(mresult)) == NULL) {
    DPRINTF (("something went wrong fetching the row\n"));
    mysql_free_result(mresult);
  } else {
    do {
      MOD_MYSQL_RR * ns_rr = (MOD_MYSQL_RR *)g_malloc0(sizeof(dnsrrrec));

      if (1 != sscanf (mrow[0], "%hu", &ns_rr->mmr_type)) {
	ns_rr->mmr_type = DT_NS;
      }

      if (1 != sscanf (mrow[1], "%hu", &ns_rr->mmr_class) 
	  || !ns_rr->mmr_class) {
	ns_rr->mmr_class = DC_IN;
      }

      if (1 != sscanf (mrow[2], "%d", &ns_rr->mmr_ttl) || !ns_rr->mmr_ttl) {
	ns_rr->mmr_ttl = mmc_ptr->mmc_soa_minimum; 
      }
      
      ns_rr->mmr_rdata_length = 
	dns_types_pack_NS (NULL, &ns_rr->mmr_rdata, zone_name, mrow[3]);

      mmc_ptr->mmc_auth_rrs = g_slist_prepend (mmc_ptr->mmc_auth_rrs, 
					       (gpointer) ns_rr);
      DPRINTF(("%s: Got NS %s!\n", ROUTINE, mrow[3]));
    } while ((mrow = mysql_fetch_row(mresult)) != NULL);
  }

#if I_KNEW_WHAT_TO_DO_HERE

  /* lookup A record for each NS record */
  /* Well, crud, I need to loop through each NS record up above
     and try to return an 'A' record for each one, but there
     is no guarantee the A records are going to be in this zone!

     One idea, use dns_build_findaddit(), but that only checks
     the Cached RR's/mod_stdddb RRs in the database.
  */
  for (curr_entry = mmc_ptr->mmc_auth_rrs;
       curr_entry;
       curr_entry = curr_entry->next) {
    
    g_snprintf (query, sizeof(query), 
		"select type, ttl, class, rdata_char, rdata_blob\n"
		" from %s\n"
		"where type=%d and domain_name='%s'",
		table_name, DT_A, crud)

  if (query_db(query, hnd, zone_name) != 0){
    DPRINTF ((ROUTINE ": query for DT_A records failed\n"));
  } else if ((mresult = mysql_store_result(&hnd->mmh_mysql_hnd)) == 0) {
    DPRINTF(("store result: %s (%d)\n", mysql_error(&hnd->mmh_mysql_hnd),
	     mysql_errno(&hnd->mmh_mysql_hnd)));
  } else if (mysql_num_rows(mresult) <= 0) {
    /* need err_warn */
    err_warn ("%s: %s; zone %s No NS records found\n", 
	       MODULE_NAME_STRING, ROUTINE, zone_name);
    DPRINTF ((ROUTINE ": TODO - set some defaults for missing SOA(?!?)\n"));
  } else if ((mrow = mysql_fetch_row(mresult)) == NULL) {
    DPRINTF (("something went wrong fetching the row\n"));
    mysql_free_result(mresult);
  } else {
    do {
      MOD_MYSQL_RR * a_rr = (MOD_MYSQL_RR *)g_malloc0(sizeof(dnsrrrec));

      if (1 != sscanf (mrow[0], "%hu", &a_rr->mmr_type)) {
	a_rr->mmr_type = DT_A;
      }

      if (1 != sscanf (mrow[1], "%hu", &a_rr->mmr_class) 
	  || !a_rr->mmr_class) {
	a_rr->mmr_class = DC_IN;
      }

      if (1 != sscanf (mrow[2], "%d", &a_rr->mmr_ttl) || !a_rr->mmr_ttl) {
	a_rr->mmr_ttl = mmc_ptr->mmc_soa_minimum; 
      }
      
      a_rr->mmr_rdata_length = 
	dns_types_pack_A (NULL, &a_rr->mmr_rdata, mrow[3]);
      mmc_ptr->mmc_addit_rrs = g_slist_prepend (mmc_ptr->mmc_addit_rrs, 
						(gpointer) a_rr);

      DPRINTF(("%s: Got A rec %s!\n", ROUTINE, mrow[3]));
    } while ((mrow = mysql_fetch_row(mresult)) != NULL);
  }
#endif

  return (mmc_ptr);
#undef ROUTINE
} /* end cache_auth_and_addit() */

/***************************************************************************
 ***************************************************************************
 **			Exported Functions
 ***************************************************************************
 ***************************************************************************/

/*
 * mod_mysql_answer
 *
 * Description: This function takes a query packet, queries the
 *              mysql database, and creates a reply in the build
 *              structure.
 * 
 * Input:       dnsqpkt - Query packet
 * Output:      build   - Response to return
 * 
 */
int
mod_mysql_answer (dnspktrec * dnsqpkt, dnsbuildrec * build)
{
#define ROUTINE "mod_mysql_answer ()"
  gchar domain_name[DNS_MAXNAMELENGTH + 1],  /* domain being queried w/o zone*/
    full_domain_name[DNS_MAXNAMELENGTH + 1],  /* domain being queried w/ zone*/
    query [QUERYLEN+(2*DNS_MAXNAMELENGTH)+1], /* SQL query to send    */
    table_name[DNS_MAXNAMELENGTH + 1],       /* name of table  in db */
    type_query [QUERYLEN],		      /* handle DT_ANY        */
    *rdata_char,	/* rdata_char column returned from mysql query */
    *rdata_blob,	/* rdata_blob column returned from mysql query */
    *zone_name = dnsqpkt->qru.query.zone->zonename,
    			/* convenience ptr to zonename field in query  */
    ipv4_string[16];
  guchar
    *rdata_buf;		/* buffer to store RDATA for RR */
  MOD_MYSQL_CACHE
    *mmc_ptr = (MOD_MYSQL_CACHE *)dnsqpkt->qru.query.zone->mdata; 
  /* cached information about this zone            */
  MOD_MYSQL_HND 
    *hnd = mmc_ptr->mmc_hnd;   /* Private module context for this zone */
  MYSQL_RES 
    *mresult;	        /* handle for status returned from query */
  MYSQL_ROW mrow;       /* handle for data returned from query */
  uint 
    *column_lengths,	/* an array of the lengths of columns in mrow*/
    row_number = 0;	/* counter for # of rows returned from mysql */
  int
    error_return = DE_NOERROR, /* Return value from the function  */
    rrs_found = 0,		/* a flag to indicate some rr's were found */
    addit_needed = 0;	/* flag to note an A record needs to be returned */
  GSList
    *curr_entry;	/* used to iterate through a GSList */
  int
    cached_time;	/* how long has the cache been valid? */
/* ----------------------------------------------------------------------- */
  DPRINTF(("%s: Starting. query=%s\n", ROUTINE, dnsqpkt->query.name));

  CHECK_MMC(mmc_ptr);
  CHECK_MMH(hnd);

  /* Yes, you can tell where the packet originated from inside a module */
  DPRINTF (("%s: Packet came from %s port %d\n",
	    ROUTINE,
	    at_inet4_ntop ((guchar *) & dnsqpkt->ip.s_addr, ipv4_string,
			   sizeof (ipv4_string)), dnsqpkt->port));

  /* NOTE: The zone context value for this zone can be stored in 
     dnsqpkt->qru.query.zone->mdata
   */

  /* We don't know anything other than the Internet class */
  if ((dnsqpkt->query.class != DC_IN)
      && (dnsqpkt->query.class != DC_ANY)) {
    DPRINTF (("%s: Unsupported class %d\n", ROUTINE, dnsqpkt->query.class));
    return (DE_NXDOMAIN);
  }

  /* If this is an ANY query, get all the domain names that match.
     Otherwise, just fetch a specific record type.
   */
  if (dnsqpkt->query.type != DT_ANY) {
    g_snprintf (type_query, sizeof(type_query),
		"AND type=%d",dnsqpkt->query.type);
  } else {
    type_query[0] = 0;
  }


  /* Check to see if the cached SOA and NS records need to be reloaded
   */
  cached_time = time(NULL) - mmc_ptr->mmc_cache_time;

  if (cached_time > CACHE_EXPIRATION_SECS 
      || cached_time > mmc_ptr->mmc_soa_minimum) {
    MOD_MYSQL_CACHE
      *new_mmc_ptr; 
    
    DPRINTF(("%s: Cached time expired. Re-fetching zone metadata\n", ROUTINE));

    new_mmc_ptr = cache_auth_and_addit(hnd, zone_name);

    /* If the re-caching was sucessful, reset the values in
       the zone structure.
    */
    if(new_mmc_ptr) {
      del_mmc(mmc_ptr);
      dnsqpkt->qru.query.zone->mdata = (void *)new_mmc_ptr;
      mmc_ptr = new_mmc_ptr;
    }
  }

  /* Strip off the domain and period */

  /* Copy over the name */
  strncpyzt (domain_name, dnsqpkt->query.name, DNS_MAXNAMELENGTH);
  strncpyzt (full_domain_name, dnsqpkt->query.name, DNS_MAXNAMELENGTH);

  /* Add the SOA information to the return IF requested specifically
     or a query type of ANY was asked for just the domain
   */
  if (!strcmp (zone_name,full_domain_name)
       && (DT_SOA == dnsqpkt->query.type || (DT_ANY == dnsqpkt->query.type))) {

    /* an SOA always has 0 TTL as per section 3.2.1 of rfc1035 */
    dns_build_add (build, DNS_BUILD_ANSWER, zone_name, 
		   0, DC_IN,  DT_SOA, 
		   (void *) mmc_ptr->mmc_soa_rdata);
    rrs_found++;
  }


  /* Now, strip off the zone name from the domain name in the query */
  if (!strcmp (domain_name, zone_name)) {
    domain_name[0]=0;
  } else {
    domain_name[strlen (domain_name) - strlen (zone_name) - 1] = 0;
  }

  g_snprintf ( query, sizeof(query), "select type, ttl, class,"
	    "rdata_char, rdata_blob"
	    " from %s "
	    " where domain=\"%s\" %s",
	    make_table_name (zone_name, table_name),
	    domain_name, type_query);

  DPRINTF (("Query is %s\n", query));

  if (query_db (query, hnd, zone_name) != 0) {
    return (DE_SERVFAIL);
  }

  if (!(mresult = mysql_store_result (&hnd->mmh_mysql_hnd))) {
    DPRINTF (("store result: %s (%d)\n", mysql_error (&hnd->mmh_mysql_hnd),
	      mysql_errno (&hnd->mmh_mysql_hnd)));
    return (DE_SERVFAIL);
  }

  if (mysql_num_rows (mresult) <= 0) {
    DPRINTF (("%s: %s; zone %s no rows returned from query\n",
	       MODULE_NAME_STRING, ROUTINE, zone_name));

    /* Be nice - free the result */
    mysql_free_result (mresult);

    /* If no RRs have been found yet, return an error, 
       BUT, it could be that an NS or SOA query (or an ANY query)
       has found RRs, even if there aren't any in the table.
    */
    if (rrs_found == 0)
      return (DE_NXDOMAIN);
  }

  /* Loop through all of the returned results and add replies
     to the build structure.
   */
  row_number = 0;
  while ((mrow = mysql_fetch_row (mresult)) != NULL) {
    int ttl = 0,		/* time to live to return */
      class = 0,		/* class to return (RED HERRING) */
      type = 0;			/* type to return */

    column_lengths = mysql_fetch_lengths(mresult);

    DPRINTF (("Row %d: \n", row_number));
    row_number++;

    if ((sscanf (mrow[0], "%d", &type) != 1) || (type <1 || type > 255)) {
      err_warn("%s: %s; zone %s: type field not valid (%s) expected integer between 1 & 255 for query %s.\n",
	       MODULE_NAME_STRING, ROUTINE, zone_name,
	       mrow[0],
	       query);
      error_return = DE_NXDOMAIN;

      break;
    }

    /* Grab the fields out of the mysql query so that we can build
       a DNS reply packet from them depending on the type of query.
     */
    if ((sscanf (mrow[1], "%d", &ttl) != 1) || ttl <= 0) {
      /* NOTE: What really needs to be done here is to get the
         TTL from the SOA record!!! 
       */
      ttl = mmc_ptr->mmc_soa_minimum;
    }

    if (sscanf (mrow[2], "%d", &class) != 1) {
      /* Set DC_IN as a default... As if we supportted any other class!!! */
      class = DC_IN;
    }

    rdata_char = mrow[3];
    rdata_blob = mrow[4];

    /* reset the rdata buffer pointer to NULL to tell dns_types_pack_XXX to
       auto-allocate memory.
     */
    rdata_buf = NULL;

    /* Build the reply depending on the type of resource record to return */
    switch (type) {
    case DT_A:
      {
	if (dns_types_pack_A(build, &rdata_buf, rdata_char) == -1) {
	  err_warn("%s: %s; zone %s IPv4 address for A Record (%s) not valid.\n",
		   MODULE_NAME_STRING, ROUTINE, zone_name,
		   rdata_char);
	  error_return = DE_NXDOMAIN;
	  break;
	}
	dns_build_add (build, DNS_BUILD_ANSWER, full_domain_name, ttl, DC_IN,
		       DT_A, (void *) rdata_buf);
	break;
      }
    case DT_AAAA:
      {
	if (dns_types_pack_AAAA(build, &rdata_buf, rdata_char) == -1) {
	  err_warn("%s: %s; zone %s IPv6 address for AAAA Record (%s) not valid.\n",
		   MODULE_NAME_STRING, ROUTINE, zone_name,
		   rdata_char);
	  error_return = DE_NXDOMAIN;
	  break;
	}
	dns_build_add (build, DNS_BUILD_ANSWER, full_domain_name, ttl, DC_IN,
		       DT_AAAA, (void *) rdata_buf);
	break;
      }

    case DT_NS:
    case DT_CNAME:
      addit_needed = 1;
    case DT_MD:
    case DT_MF:
    case DT_MB:
    case DT_MG:
    case DT_MR:
    case DT_PTR:
      {
	if (dns_types_pack_1_domain_name (build, &rdata_buf, zone_name,
					  rdata_char) == -1) {
	  err_warn("%s: %s; zone %s Domain Name for %s Record (%s) not valid.\n", 
		   MODULE_NAME_STRING, ROUTINE, zone_name,
		   dns_dt_to_string(dnsqpkt->query.type),
		   rdata_char);
	  
	  error_return = DE_NXDOMAIN;
	  break;
	}
	dns_build_add (build, DNS_BUILD_ANSWER, 
		       full_domain_name, ttl, DC_IN,type, rdata_buf);
	break;
      }
    case DT_NULL:
      {
	char * rdata_buf = dns_build_alloc_data(build, column_lengths[4]);
	
	/* NOTE: DENTS cannot handle this properly because 
	   it doesn't preserve the length of the item stored 
	   in rdata_buf.
	 */
	memcpy (rdata_buf, rdata_blob, column_lengths[4]);
	dns_build_add (build, DNS_BUILD_ANSWER,
		       full_domain_name, ttl, DC_IN, type, 
		       rdata_buf);
	break;
      }
    case DT_WKS:
      {
	NOT_IMPL ("DT_WKS");
	break;
      }
    case DT_HINFO:
    case DT_ISDN:
      {
	if (dns_types_pack_2_character_strings(build, &rdata_buf, 
					       rdata_char, rdata_blob) == -1) {
	  err_warn("%s: %s; zone %s Character String %s or %s for Record type %s not valid.\n",
		   MODULE_NAME_STRING, ROUTINE, zone_name,
		   rdata_char, rdata_blob,
		   dns_dt_to_string(dnsqpkt->query.type));

	  error_return = DE_NXDOMAIN;
	  break;
	}
	dns_build_add (build, DNS_BUILD_ANSWER, full_domain_name, ttl, DC_IN,
		       type, rdata_buf);
	break;
      }
    case DT_MINFO:
    case DT_RP:
      {
	if (dns_types_pack_2_domain_names(build, &rdata_buf, zone_name,
					  rdata_char, rdata_blob) == -1) {
	  err_warn("%s: %s; zone %s Domain Name %s or %s for Record type %s not valid.\n",
		   MODULE_NAME_STRING, ROUTINE, zone_name,
		   rdata_char, rdata_blob,
		   dns_dt_to_string(dnsqpkt->query.type));

	  error_return = DE_NXDOMAIN;
	  break;
	}
	dns_build_add (build, DNS_BUILD_ANSWER, full_domain_name, ttl, DC_IN,
		       type, rdata_buf);
	break;
      }
    case DT_MX:
      addit_needed = 1;
    case DT_AFSDB:
      {
	int tmp_int;
	guint16 preference;

	if (sscanf (rdata_char, "%d", &tmp_int) != 1) {

	  err_warn
	    ("%s: %s; zone %s Data for MX/AFSDB PREFERENCE(%s %s) out of range 0-65535.\n",
	     MODULE_NAME_STRING, ROUTINE, zone_name,
	     rdata_char);
	  error_return = DE_NXDOMAIN;
	  break;
	}

	/* copy the preference value first */
	preference = tmp_int & 0xFFFF;
	
	if (dns_types_pack_uint16_domain(build, &rdata_buf, preference,
					 zone_name, rdata_blob) == -1) {
	  err_warn
	    ("%s: %s; zone %s Domain Name %s or preference %d  for Record type %s not valid.\n",
	     MODULE_NAME_STRING, ROUTINE, zone_name,
	     rdata_blob, preference,
	     dns_dt_to_string(dnsqpkt->query.type));

	  error_return = DE_NXDOMAIN;
	  break;
	}
	dns_build_add (build, DNS_BUILD_ANSWER, full_domain_name, ttl, DC_IN,
		       type, rdata_char);
	break;
      }
    case DT_TXT:
    case DT_NSAP:
      {
	if (dns_types_pack_1_character_string(build, &rdata_buf,
					      rdata_char) == -1) {
	  err_warn("%s: %s; zone %s Character String for %s Record (%s) not valid.\n", 
		   MODULE_NAME_STRING, ROUTINE, zone_name,
		   dns_dt_to_string(dnsqpkt->query.type),
		   rdata_char);
	  
	  error_return = DE_NXDOMAIN;
	  break;
	}

	dns_build_add (build, DNS_BUILD_ANSWER, full_domain_name, ttl, DC_IN,
		       DT_TXT, rdata_char);
	break;
      }

      /* the rest aren't supported (yet) */
    default:
      {
	char case_number[10];

	sprintf (case_number, "%d", dnsqpkt->query.type);
	NOT_IMPL (case_number);
	break;
      }
    } /* end switch */
  } /* end while */

  /* Be nice - free the result (if we got one at all) */
  if (mresult)
    mysql_free_result(mresult);

  /* Fill out the Authority portion of the response.
     These are the NS records for the zone that were 
     cached in cache_auth_and_addit()
   */
  for (curr_entry = mmc_ptr->mmc_auth_rrs;
       curr_entry;
       curr_entry = curr_entry->next) {
    MOD_MYSQL_RR * rr_ptr = (MOD_MYSQL_RR *)curr_entry->data;

    dns_build_add (build, DNS_BUILD_NS, zone_name, 
		   MAX(mmc_ptr->mmc_soa_minimum, rr_ptr->mmr_ttl), 
		   DC_IN, rr_ptr->mmr_type, rr_ptr->mmr_rdata); 
  }

  /* Fill out the Additional portion of the response.  These
     are the A records for any of the RRs returned above that contain
     domain names in the rdata section.  We always need to return any 
     A records we have for the NS records in the authority section.

     Unfortunately, the A records that need to be returned may be either
     inside or outside of this zone.  I'm not sure how to handle this.
  */
  if (addit_needed) {
    DPRINTF (("TODO - If an MX, CNAME, or NS record is returned, we need to add A records to the additional section."));
  }

  return (error_return);

#undef ROUTINE
}				/* end mod_mysql_answer() */


/*
 * module_dns_zone_add
 *
 * Adds a zone. I'm not sure what this function is supposed to do.
 * I think it is a leftover.
 *
 * -EZA
 *
 */
MODULE_FUNC (int, dns_zone_add, (dnszonerec * zone))
{
  DPRINTF (("mod_mysql_dns_zone_add: Called\n"));

  return (DE_NOTIMP);
}

/*
 * module_dns_zone_delete
 *
 *  Delete's a zone
 *
 * FIXME: what are the valid return values from this function???
 *
 */
MODULE_FUNC (int, dns_zone_delete, (dnszonerec * zone))
{
#define ROUTINE "mod_mysql_eza_dns_zone_delete ()"
    MOD_MYSQL_CACHE
      *mmc_ptr;
    MOD_MYSQL_HND
      *hnd;
      
  DPRINTF (("%s: Called\n", ROUTINE));
  mmc_ptr = (MOD_MYSQL_CACHE *)zone->mdata;
  CHECK_MMC(mmc_ptr);
  hnd = mmc_ptr->mmc_hnd;
  CHECK_MMH(hnd);

  del_mmc(mmc_ptr);
  del_mmh(hnd);
  zone->mdata = NULL;

  return (0);

#undef ROUTINE
}

/*
 * module_mysql_zone_load
 *
 * A new zone is requested to be loaded.  This doesn't really
 * have to do anything - it could just verify that the table is 
 * there.  Or it could connect to the databa se
 *
 *
 */
MODULE_FUNC (int, dns_zone_load, (dnszonerec * zone))
{
#define ROUTINE "module_mysql_zone_load ()"

  char *host_name,		/* host name of the DB to connect to */
   *db_name,			/* database name of DB to connect to */
   *user_name,			/* MYSQL user name */
   *password,			/* MYSQL password  */
    *zone_name,			/* name of this zone */
    table_name[DNS_MAXNAMELENGTH + 1];   /* name of table  in db */  
  int i;			/* loop control variable. */
  MOD_MYSQL_CACHE *mmc_ptr;	/* structure containing cached zone info */
  MOD_MYSQL_HND *hnd;		/* structure containing open database handle */
  MYSQL_RES 
    *mresult;		  	/* handle for status returned from query */
  /* ----------------------------------------------------------------------- */

  DPRINTF (("%s: Starting %d args", ROUTINE, zone->argc));

  /* Loop through and print arguments */
  for (i = 0; i < zone->argc; i++) {
    DPRINTF ((" arg %d: %s", i, zone->argv[i]));
  }
  DPRINTF (("\n"));


  zone_name = zone->zonename;

  if (zone->argc < 4) {

    err_warn ("%s: %s; zone %s expected 4 arguments to module <host_name> <db_name> <user_name> <password>", 
	      MODULE_NAME_STRING, ROUTINE, zone_name);
    return (ERR_DNS_RDB_PARSEERROR);
  }

  host_name = zone->argv[0];
  db_name = zone->argv[1];
  user_name = zone->argv[2];
  password = zone->argv[3];

  /* look for an open handle */
  hnd = find_mmh (host_name, db_name, user_name);

  if (hnd) {
    /* increment the reference count. */
    hnd->mmh_ref_cnt++;
  } else {
    /* allocate & initialize a new handle */
    hnd = (MOD_MYSQL_HND *) g_malloc0 (sizeof (MOD_MYSQL_HND));
    hnd->mmh_magic = MMH_MAGIC;
    hnd->mmh_host_name = g_strdup (host_name);
    hnd->mmh_db_name = g_strdup (db_name);
    hnd->mmh_user_name = g_strdup (user_name);
    hnd->mmh_password = g_strdup (password);
    hnd->mmh_ref_cnt = 1;
    hnd->mmh_mysql_hnd_init = 0;
  }

  /* Attempt to connect to the database */

  if (connect_to_db(hnd, zone_name)) {
    del_mmh(hnd);
    return (ERR_DNS_RDB_NOSUCHFILE);
  }

  /* look for a table named 'res_<zone_name>  in the database*/
  make_table_name (zone->zonename, table_name);

  if (!(mresult = mysql_list_tables (&hnd->mmh_mysql_hnd, table_name))) {

      err_warn ("%s: %s; zone %s mysql_list_tables(%s): %s (%d)\n", 
	       MODULE_NAME_STRING, ROUTINE, zone_name,
	       table_name,
	       mysql_error(&hnd->mmh_mysql_hnd),
	       mysql_errno(&hnd->mmh_mysql_hnd));
      return (ERR_DNS_RDB_NOSUCHFILE);
  }

  /* Check to see if any tabgle exists with this name */
  if (mysql_num_rows(mresult) <=0) {
    err_warn ("%s: %s; zone %s: no rows returned from query for %s\n", 
	      MODULE_NAME_STRING, ROUTINE, zone_name,
	      table_name);
    mysql_free_result (mresult);
    return (ERR_DNS_RDB_NOSUCHFILE);
  }

  /* Free the result when we are done */
  mysql_free_result (mresult);
  hnd->mmh_mysql_hnd_init = 1;

  mmc_ptr = cache_auth_and_addit(hnd, zone_name);

  if (!mmc_ptr) {
    DPRINTF (("%s: cache_auth_and_addit() failed!\n", ROUTINE));
    del_mmh(hnd);
    return (ERR_DNS_RDB_PARSEERROR);
  }
  
  CHECK_MMC(mmc_ptr);

  DPRINTF (("%s: MSQL login OK. Table %s exists.\n", ROUTINE, table_name));

  /* We can assign a context value for this zone to zone->mdata */
  zone->mdata = (void *) mmc_ptr;

  return (ERR_DNS_RDB_NOERROR);

#undef ROUTINE
} /* end dns_zone_load () */


/*
 * module_dns_zone_reload
 *
 *  Reload's a zone.
 *
 * Note that the names of the return values from this function are 
 * leftover from the * mod_stdddb implementation (ERR_DNS_RDB_XXXX)
 * which are due to be renamed.
 */
MODULE_FUNC (int, dns_zone_reload, (dnszonerec * zone))
{
    MOD_MYSQL_CACHE
      *mmc_ptr,
      *new_mmc_ptr;
    MOD_MYSQL_HND
      *hnd;
      
    mmc_ptr = (MOD_MYSQL_CACHE *)zone->mdata;
    CHECK_MMC(mmc_ptr);
    hnd = mmc_ptr->mmc_hnd;
    CHECK_MMH(hnd);
    
    new_mmc_ptr = cache_auth_and_addit(hnd, zone->zonename);

    /* If the re-caching was sucessful, reset the values in
       the zone structure.
    */
    if(new_mmc_ptr) {
      del_mmc(mmc_ptr);
      zone->mdata = (void *)new_mmc_ptr;
      mmc_ptr = new_mmc_ptr;
      return (ERR_DNS_RDB_NOERROR);
    }
    
    return (ERR_DNS_RDB_NOSUCHFILE);

} /* end dns_zone_reload() */


/*
 * module_init
 *
 *  All we do is just print a banner
 */
MODULE_FUNC (int, init, (void))
{
  DPRINTF (("mod_mysql_init: version %d.%d.%d by Eric Z. Ayers\n",
	    MODULE_VER_MAJOR, MODULE_VER_MINOR, MODULE_VER_MICRO));

  return 0;
}

/*
 * module_destroy
 *
 * Called when the module is unloaded. Always fter all zones have
 * stopped using it
 *
 * Note that this doesn't free the 'MOD_MYSQL_CACHE' pointers.  I'm not 
 * sure how dents behaves - will it call zone delete before it calls
 * module destroy?
 */
MODULE_FUNC (void, destroy, (void))
{
#define ROUTINE "mod_mysql_destroy ()"
  MOD_MYSQL_HND *hnd;		/* for iterating through linked list */

  DPRINTF (("%s: Starting\n", ROUTINE));

  /* go through the list of open database handles and mark them all unused */
  for (hnd = link_first (&gbl_hnd_list);
       hnd; hnd = link_next (hnd, hnd->mmh_ll)) {
    hnd->mmh_ref_cnt = 0;
  }

  /* go through the list of database handles and delete them */
  while ((hnd = link_first (&gbl_hnd_list)) != NULL) {
    del_mmh (hnd);
  }

  DPRINTF (("%s: freed all resources\n", ROUTINE));

#undef ROUTINE
}				/* end mod_mysql_destroy() */

/***************************************************************************
 ***************************************************************************
 ** 		DENTS module black voodoo magic
 ***************************************************************************
 ***************************************************************************/

/*
 * Module compile support code
 */

/* Each answering function in order of desired execution. The list MUST */
/*  be NULL terminated */
MODULE_DNS_ANSWER = {
mod_mysql_answer, NULL};

/*
 * Exported symbols
 */
MODULE_SYMS (MODULE_SYMS_INIT	/* module_init() */
	     MODULE_SYMS_DESTROY	/* module_destroy() */
	     MODULE_SYMS_DNS	/* Mandatory DNS symbols */
	     /* module_dns_zone_add() */
	     /* module_dns_zone_delete() */
	     /* module_dns_answer() */
	     MODULE_SYMS_DNS_ZONE_RELOAD	/* module_dns_zone_reload() */
	     MODULE_SYMS_DNS_ZONE_LOAD	/* module_dns_zone_load() */
  )
#else

/* This is just a stub - the test for MYSQL failed from the configure script
 * so WITH_MYSQL  did not get defined.
 */
static int mod_mysql_disabled = 1;
