/* mod_perl.c - calls perl in order to resolve requests.

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Perl-related things. */
#include <EXTERN.h>
#undef PACKAGE			/* Automake wants '"dents"' but perl.h wants
				   '270'. */
#include <perl.h>
#undef STATIC			/* We need STATIC as an enum-value in
				   module.h. */

/* Add system headers here */
#include <glib.h>
#include <signal.h>
#include <sys/types.h>

/* DENTS includes */
#define MODULE_NAME mod_perl
#define MODULE_VER_MAJOR 1
#define MODULE_VER_MINOR 0
#define MODULE_VER_MICRO 1
#include "dns.h"
#include "module_syms.h"
#include "dns_module_syms.h"
#include "link.h"
#include "addr_tools.h"

static PerlInterpreter *perl;
char *func = NULL;

/* 
 * mod_perl_answer
 *
 * Description: This function takes a query packet, passes it off to 
 *              a perl script (argv[1]), and creates a reply in the build
 *              structure.
 * 
 * Input:       dnsqpkt - Query packet
 * Output:      build   - Response to return
 * 
 */
int
mod_perl_answer (dnspktrec * dnsqpkt, dnsbuildrec * build)
{
#define ROUTINE "mod_perl_answer ()"
#define QUERYLEN	256
  dSP;
  HV *qpkt;
  HV *apkt;
  SV *ip_sv;
  SV *query_class;
  SV *query_type;
  SV *query_name;
  SV *qhr;
  SV *ahr;
  char *ip = g_new (char, 16);
  char *func = NULL;
  int count;
  qpkt = newHV ();
  apkt = newHV ();

  at_inet4_ntop ((guchar *)&(dnsqpkt->ip.s_addr), ip, 16);
  ip_sv = newSVpv (ip, strlen (ip));
  query_class = newSViv (dnsqpkt->query.class);
  query_type = newSViv (dnsqpkt->query.type);
  query_name = newSVpv (dnsqpkt->query.name, strlen (dnsqpkt->query.name));

  hv_clear (qpkt);
  hv_clear (apkt);

  hv_store (qpkt, "ip", strlen ("ip"), ip_sv, 0);
  hv_store (qpkt, "class", strlen ("class"), query_class, 0);
  hv_store (qpkt, "type", strlen ("type"), query_type, 0);
  hv_store (qpkt, "name", strlen ("name"), query_name, 0);

  qhr = newRV ((SV *) qpkt);
  ahr = newRV ((SV *) apkt);

  ENTER;

  SAVETMPS;

  PUSHMARK (SP);
  XPUSHs (sv_2mortal (qhr));
  PUTBACK;

  count = perl_call_pv (func, G_SCALAR);

  SPAGAIN;

  ahr = POPs;
  PUTBACK;
  FREETMPS;
  LEAVE;

  /* ----------------------------------------------------------------------- */
  return 0;
}

/* 
 * Exported functions
 */

/* 
 * module_dns_zone_add
 *
 *  Add's a zone
 */
MODULE_FUNC (int, dns_zone_add, (dnszonerec * zone))
{
  DPRINTF (("mod_perl_dns_zone_add: Called\n"));

  return (0);
}

/* 
 * module_dns_zone_delete
 *
 *  Delete's a zone
 */
MODULE_FUNC (int, dns_zone_delete, (dnszonerec * zone))
{
  DPRINTF (("mod_perl_dns_zone_delete: Called\n"));

  return (DE_NOTIMP);
}

/* 
 * module_perl_zone_load
 *
 */
MODULE_FUNC (int, dns_zone_load, (dnszonerec * zone))
{

  perl = perl_alloc ();
  perl_construct (perl);

  perl_parse (perl, NULL, zone->argc, zone->argv, NULL);
  func = (char *) malloc (strlen (zone->argv[0]));
  strcpy (func, zone->argv[0]);
  return 0;
}				/* end dns_zone_load () */

/* Reload's a zone. */
MODULE_FUNC (int, dns_zone_reload, (dnszonerec * zone))
{
  return (DE_NOTIMP);
}

/* All we do is just print a banner. */
MODULE_FUNC (int, init, (void))
{
  DPRINTF (("mod_perl_init: version %d.%d.%d by K. William Young\n",
	    MODULE_VER_MAJOR, MODULE_VER_MINOR, MODULE_VER_MICRO));

  return 0;
}

/* 
 * module_destroy
 *
 *  Called when the module is unloaded. Always fter all zones have
 * stopped using it
 */
MODULE_FUNC (void, destroy, (void))
{

  perl_destruct (perl);
  perl_free (perl);

}

/***************************************************************************
 ***************************************************************************
 ** 		DENTS module black voodoo magic
 ***************************************************************************
 ***************************************************************************/

/* 
 * Module compile support code
 */

/* Each answering function in order of desired execution. The list MUST */
/* be NULL terminated */
MODULE_DNS_ANSWER = {
mod_perl_answer, NULL};

/* 
 * Exported symbols
 */
MODULE_SYMS (MODULE_SYMS_INIT	/* module_init() */
	     MODULE_SYMS_DESTROY	/* module_destroy() */
	     MODULE_SYMS_DNS	/* Mandatory DNS symbols */
	     /* module_dns_zone_add() */
	     /* module_dns_zone_delete() */
	     /* module_dns_answer() */
	     MODULE_SYMS_DNS_ZONE_RELOAD	/* module_dns_zone_reload() */
	     MODULE_SYMS_DNS_ZONE_LOAD	/* module_dns_zone_load() */
  )
