/* mod_stddb.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/*
 * mod_stddb.c
 *
 *  Module for the standard in-memory database
 */
#include "dns.h"

/* Basic info on the module, used later */
#define MODULE_NAME	mod_stddb

#define MODULE_VER_MAJOR 0
#define MODULE_VER_MINOR 0
#define MODULE_VER_MICRO 1

#include "module_syms.h"
#include "dns_module_syms.h"

/* Database containing all of the RR's */
static dbrootrec db;

/*
 * mod_stddb_retsoa
 *
 *  This is only used when we get a query about a zone we're authoritative
 * for. It returns the SOA which is now the standard for NXDOMAIN's
 */
int
mod_stddb_retsoa (dnspktrec * dnsqpkt, dnsbuildrec * build)
{
  dbnoderec *node;
  dnsdbrrrec *rr;

  /* Lala. Find the node */
  if ((node = dns_db_find (&db, dnsqpkt->qru.query.zone->zonename))
      == NULL) {
    err_warn ("mod_stddb_retsoa: dns_db_find returned NULL!");
    return (DE_SERVFAIL);
  }

  /* Match the RR */
  if ((rr = dns_db_rr_findmatch (node, NULL, dnsqpkt->query.class, DT_SOA))
      == NULL) {
    err_warn ("mod_stddb_retsoa: umm, dns_db_matchrr returned NULL!");
    return (DE_SERVFAIL);
  }

  /* Throw it into the build structure */
  dns_build_addrr (build, DNS_BUILD_NS, rr);

  return (DE_NXDOMAIN);
}

/*
 * mod_stddb_db
 *
 *  Finds all of the information related to the original query. If we don't
 * find anything, we'll return DE_NOINFO. If everything is ok, we'll return
 * DE_NOERROR or if there's an error, we'll return the appropriate DE_ error
 * code
 */
int
mod_stddb_db (dnspktrec * dnsqpkt, dnsbuildrec * build)
{
  int rc;
  dbnoderec *node;

  if ((node = dns_db_find (&db, dnsqpkt->query.name)) == NULL) {
    DPRINTF (("mod_stddb_db: dns_db_find returned NULL on %s\n",
	      dnsqpkt->query.name));
    return (DE_NOINFO);
  }

  if ((rc = dns_build_findans_and_pack (&db, node, &dnsqpkt->query, build)) !=
      DE_NOERROR) return (rc);

  dns_build_findauth_and_pack (&db, node, &dnsqpkt->query, build);

  dns_build_findaddit_and_pack (&db, node, &dnsqpkt->query, build);

  return (DE_NOERROR);
}

/*
 * Exported functions
 */

/*
 * module_dns_zone_Add
 *
 *  Add's a zone
 */
MODULE_FUNC (int, dns_zone_add, (dnszonerec * zone))
{
  printf ("mod_stddb_add: Called\n");

  return (0);
}

/*
 * module_dns_zone_delete
 *
 *  Delete's a zone and free's all associated memory
 */
MODULE_FUNC (int, dns_zone_delete, (dnszonerec * zone))
{
  printf ("mod_stddb_delete: Called\n");

  return (DE_NOTIMP);
}

/*
 * module_dns_zone_reload
 *
 *  Reload's an already loaded zone
 */
MODULE_FUNC (int, dns_zone_reload, (dnszonerec * zone))
{
  printf ("mod_stddb_reload: Not implemented, yet\n");

  return (DE_NOTIMP);
}

/*
 * module_dns_zone_load
 *
 *  Load's a zone from disk (standard RFC 883 style)
 */
MODULE_FUNC (int, dns_zone_load, (dnszonerec * zone))
{
  return (dns_rdb_read (&db, DNS_RDB_SANITYCHECK,
			zone->zonename, zone->filename));
}

/*
 * module_init
 *
 *  Print banner and init the database
 */
MODULE_FUNC (int, init, (void))
{
  DPRINTF (("mod_stddb_init: version %d.%d.%d by Johannes Erdfelt\n",
	    MODULE_VER_MAJOR, MODULE_VER_MINOR, MODULE_VER_MICRO));

  /* Init the database */
  db_init (&db);

  return (0);
}

/*
 * No module_destroy
 */

/*
 * Module compile support code
 */

/*
 * Define some symbols
 */
MODULE_DNS_ANSWER = {
mod_stddb_db, mod_stddb_retsoa, NULL};

/*
 * Exported symbols
 */
MODULE_SYMS (MODULE_SYMS_INIT
	     MODULE_SYMS_DNS
	     MODULE_SYMS_DNS_ZONE_LOAD MODULE_SYMS_DNS_ZONE_RELOAD)
