/* addr_tools.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */
#ifndef __ADDR_TOOLS_H__
#define __ADDR_TOOLS_H__



/* We need this here... maybe this should be moved to another include file */
#include <netinet/in.h>

/*
 * Do we want the "weird" ipv4 addressing method? 0.0.1.2 == 258
 * Than uncomment the following define.
 */
/* #define AT_OBSCURE_IPV4 */

/* Present a clear interface to the different addressing types */
struct at_addr
{
	int	aa_type;	/* What kind of address are we holding */

	union
	{
		struct in_addr aa_addr_in;
#ifdef HAVE_AF_INET6
		struct in6_addr aa_addr_in6;
#endif
	} aa_addr;
};

#define aa_in_addr	aa_addr.aa_addr_in
#define aa_in6_addr	aa_addr.aa_addr_in6

/*
 * at_inet4_ntop, return the packed IPv4 address in *ap as a human readable
 * string
 *
 * at_inet4_ntop(const guint8 *ap, char *cp, size_t len)
 *
 * Returns *cp on success, NULL on error, which can only happen on a
 * parameter problem
 */
char *at_inet4_ntop (const guint8 * ap, char *cp, size_t len);

/*
 * at_inet6_ntop, return the packed IPv6 address in *ap as a human readable
 * string
 *
 * at_inet6_ntop(const guint8 *ap, char *cp, size_t len)
 *
 * Returns *cp on success, NULL on error, which can only happen on a
 * parameter problem
 */
char *at_inet6_ntop (const guint8 * ap, char *cp, size_t len);

/*
 * at_inet4_pton, return the human readable IPv4 string as a packed IPv4 address
 *
 * at_inet4_pton(char *cp, guint8 *ap)
 *
 * Returns *ap on success, NULL on error, in which case *cp is not a valid
 * formatted ipv4 address
 */
guint8 *at_inet4_pton (char *cp, guint8 * ap);

/*
 * at_inet6_pton, return the human readable IPv6 string as a packed IPv6 address
 *
 * at_inet6_pton(char *cp, guint8 *ap)
 *
 * Returns *ap on success, NULL on error, in which case *cp is not a valid
 * formatted ipv6 address
 */
guint8 *at_inet6_pton (char *cp, guint8 * ap);

/* at_inet4_aton, convert human readable IPv4 address to packed in_addr
 * structure. It is just an interface to at_inet4_pton modeled after inet_ntoa
 *
 * int at_inet4_aton(const char *cp, struct in_addr *inp)
 *
 */
int at_inet4_aton(const char *cp, struct in_addr *inp);

#ifdef HAVE_AF_INET6
/* at_inet6_aton, convert human readable IPv6 address to packed in6_addr
 * structure. It is just an interface to at_inet6_pton modeled after inet_ntoa
 * And is only provided if the host support IPv6 in it's libc
 * 
 * int at_inet6_aton(const char *cp, struct in6_addr *inp)
 * 
 */
int at_inet6_aton(const char *cp, struct in6_addr *inp);
#endif

#endif
