/* conf.c - API for configuration information.

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

/* Provides an easy to use API to some configuration information.  We
   don't have a specific configuration file format.  In fact, we can
   read a couple of different formats just fine. */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

#include "conf.h"
#include "db.h"
#include "db_rb.h"

dbrb ip_name_db;

struct ip_name predef_ipnames[] = {
  {{INADDR_ANY}, 32, 0, DB_RB_NULL, "any"},
  {{INADDR_ANY}, 32, IPMATCH_NOT, DB_RB_NULL, "none"},
  {{INADDR_LOOPBACK}, 32, 0, DB_RB_NULL, "localhost"},
  {{INADDR_LOOPBACK}, 8, 0, DB_RB_NULL, "localnets"},
};

#define NUM_IPNAMES	(sizeof(predef_ipnames) / sizeof(struct ip_name))

struct iplist *
iplist_create (void)
{
  return (struct iplist *) malloc (sizeof (struct iplist));
}

void
iplist_add (struct iplist *ipl, struct iplist_entry *iple)
{
  link_add (&ipl->e, (struct llentry *) iple);
}

struct iplist_entry *
iplist_entry_nm (struct in_addr ip, u_char length)
{
  struct iplist_entry *iple;

  iple = malloc (sizeof (*iple));
  if (!iple)
    return NULL;

  iple->type = IPL_TYPE_DIRECT;
  iple->flags = 0;

  iple->u.ipnm.a = ip;
  iple->u.ipnm.l = length;

  return iple;
}

struct iplist_entry *
iplist_entry_acl (struct iplist *ipl)
{
  struct iplist_entry *iple;

  iple = malloc (sizeof (*iple));
  if (!iple)
    return NULL;

  iple->type = IPL_TYPE_INDIRECT;
  iple->flags = 0;

  iple->u.iplist = ipl;

  return iple;
}

#ifdef DENTS_DEBUG
void print_iplist (struct iplist *ipl);
void
_print_iplist (struct iplist_entry *iple)
{
  if (!iple)
    return;

  for (; iple; iple = iple->next) {
    if (iple->flags & IPMATCH_NOT)
      printf ("NOT\n");

    switch (iple->type) {
    case IPL_TYPE_INDIRECT:
      print_iplist (iple->u.iplist);
      break;
    case IPL_TYPE_DIRECT:
      printf ("ip: %s/%d\n", inet_ntoa (iple->u.ipnm.a), iple->u.ipnm.l);
      break;
    }
  }
}

void
print_iplist (struct iplist *ipl)
{
  _print_iplist (ipl->e.begin);
}
#endif

struct iplist_entry *
iplist_name (u_char * name)
{
  struct ip_name *ipn;
  struct iplist_entry *iple;

  ipn = db_rb_find (&ip_name_db, name);
  if (!ipn)
    return NULL;

  iple = iplist_entry_nm (ipn->a, ipn->l);
  if (!iple)
    return NULL;

  iple->flags = ipn->flags;
  return iple;
}

void
conf_init (void)
{
  int i;

  db_rb_create (&ip_name_db, db_rb_cmp_stri);

  for (i = 0; i < NUM_IPNAMES; i++)
    db_rb_insert (&ip_name_db, &predef_ipnames[i].namenode,
		  &predef_ipnames[i]);
}
