%{
/* conf_nc_parser.[cy] - parses named.conf style configuration file.

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>          /* struct in_addr */
#include <sys/socket.h>         /* struct in_addr */
#include <netinet/in.h>         /* struct in_addr */
#include <arpa/inet.h>		/* inet_ntoa */
#include <stdarg.h>		/* valist, etc */
#include <stdio.h>		/* FILE, fopen, etc */

#include "conf.h"
#include "conf_nc_lexer.h"

/* Quickie macro'ss */
#define SET_STRING(x, v)	\
	if (x)			\
	  free(x);		\
	x = v;
%}

%union
{
  char *string;
  struct in_addr ipaddr;
  long num;

  struct iplist *ipl;
  struct iplist_entry *iple;
}

/* Root level options */
%token C_INCLUDE C_OPTIONS C_ZONE C_ACL C_KEY C_SERVER C_LOGGING
/* "options" directives */
%token C_DIRECTORY C_NAMED_XFER C_PID_FILE C_STATISTICS_FILE C_CHECK_NAMES
%token C_MEMSTATISTICS_FILE C_DUMP_FILE C_FAKE_IQUERY C_RECURSION
%token C_FETCH_GLUE C_NOTIFY C_HOST_STATISTICS C_DEALLOCATE_ON_EXIT
%token C_AUTH_NXDOMAIN C_MAX_TRANSFER_TIME_IN C_FIRST C_ONLY
%token C_MULTIPLE_CNAMES C_ALLOW_QUERY C_ALLOW_TRANSFER C_TRANSFERS_IN
%token C_TRANSFER_PER_NS C_TRANSFERS_OUT C_MAX_TRANSFER_TIME_IN
%token C_TRANSFER_FORMAT C_QUERY_SOURCE C_FORWARD C_FORWARDERS C_TOPOLOGY
%token C_LISTEN_ON C_CLEANING_INTERVAL C_INTERFACE_INTERVAL
%token C_STATISTICS_INTERVAL 

/* "zone" directives */
%token C_IN C_CHAOS C_HESIOD C_CSNET C_TYPE C_FILE C_MASTERS C_ALLOW_UPDATE
%token C_ALSO_NOTIFY
/* C_ALLOW_QUERY C_ALLOW_TRANSFER C_CHECK_NAMES C_NOTIFY */
/* "acl" directives */
%token C_CAN_QUERY C_CAN_AXFR
/* "key" directives */
%token C_ALGORITHM C_SECRET
/* "server" directives */
%token C_BOGUS C_TRANSFER_FORMAT C_TRANFERS C_KEYS
/* "logging" directives */
%token C_CHANNEL C_CATEGORY C_SYSLOG_ERRORS C_SYSLOG C_SEVERITY C_PARSER
%token C_DEFAULT_SYSLOG C_LAME_SERVERS C_MODERATE_DEBUG C_PRINT_TIME
%token C_PRINT_CATEGORY C_PRINT_SEVERITY C_LOAD C_NO_INFO_MESSAGES C_PANIC
%token C_EVENTLIB C_PACKET
/* constants */
%token C_YES C_NO C_IGNORE C_IF_NO_ANSWER C_MANY_ANSWERS C_MASTER C_NULL
%token C_ONE_ANSWER C_PORT C_RESPONSE C_SIZE C_SLAVE C_STACKSIZE
%token C_STUB C_TRANSFER_SOURCE C_TRANSFERS C_TRANSFERS_PER_NS
%token C_TRUE C_UNLIMITED C_VERSIONS C_WARN C_ADDRESS C_CORESIZE
%token C_DATASIZE C_DEALLOC_ON_EXIT C_DEBUG C_DEFAULT C_DYNAMIC
%token C_FAIL C_FALSE C_FILES C_HINT C_IF_NO_DOMAIN
%token EOS END_INCLUDE
%token < string > QSTRING STRING
%token < ipaddr > IPADDR
%token < num > NUMBER
%type < num > yes_or_no check_names_action transfer_format
%type < num > zone_type
%type < ipaddr > wild_addr
%type < num > wild_port poss_port port
%type < ipl > address_list
%type < iple > address_entry address_simple address_name

%%

start:command_list;

command_list:command | command_list command;

command:include | options_block EOS | zone_block EOS;

	/* We should check against /etc/services as well */
port:NUMBER {
  /* Check for valid number? */
  $$ = $1;
}
;

wild_addr:IPADDR | '*' {
  $$.s_addr = htonl (INADDR_ANY);
}
;

wild_port:port | '*' {
  $$ = htons (0);
}
;

check_names_type:C_MASTER check_names_action {
  config.cn_master = $2;
}
|C_SLAVE check_names_action {
  config.cn_slave = $2;
}
|C_RESPONSE check_names_action {
  config.cn_response = $2;
}
;

check_names_action:C_FAIL {
  $$ = CN_FAIL;
}
|C_WARN {
  $$ = CN_WARN;
}
|C_IGNORE {
  $$ = CN_IGNORE;
}
;

transfer_format:C_ONE_ANSWER {
  $$ = TRANS_ONE;
}
|C_MANY_ANSWERS {
  $$ = TRANS_MANY;
}
;

masters_ipaddr:IPADDR {
  /* Verify the IP Address */
  printf ("Adding IP %s\n", inet_ntoa ($1));
}
|STRING {
  printf ("IP name: %s\n", $1);
}
|'*' {
  printf ("Any IP\n");
}
;

masters_ipaddr_list:masters_ipaddr EOS
  | masters_ipaddr_list masters_ipaddr EOS;

address_list:address_entry EOS {
  struct iplist *ipl;

  ipl = iplist_create ();
  if ($1 != NULL)
    iplist_add (ipl, $1);
  $$ = ipl;
}
|address_list address_entry EOS {
  if ($2 != NULL)
    iplist_add ($1, $2);
  $$ = $1;
}
;

address_entry:address_simple | '!' address_simple {
  if ($2 != NULL)
    $2->flags |= IPMATCH_NOT;
  $$ = $2;
}
;

address_name:STRING {
  $$ = iplist_name ($1);
}
;

address_simple:IPADDR {
  $$ = iplist_entry_nm ($1, 32);
}
|IPADDR '/' NUMBER {
  if ($3 < 0 || $3 > 32) {
    yyerror ("subnet length does not make sense, ignoring");
    $$ = NULL;
  } else {
    $$ = iplist_entry_nm ($1, $3);
    if ($$ == NULL)
      yyerror ("address/mask error, ignoring");
  }
}
|NUMBER '/' NUMBER {
  struct in_addr ip;

  if ($1 > 255) {
    yyerror ("network does not make sense, ignoring");
    $$ = NULL;
  } else {
    if ($3 < 0 || $3 > 32) {
      yyerror ("subnet length does not make sense, ignoring");
      $$ = NULL;
    } else {
      ip.s_addr = htonl ($1 << 24);
      $$ = iplist_entry_nm (ip, $3);
      if ($$ == NULL)
	yyerror ("address/mask error, ignoring");
    }
  }
}
|address_name | '{' address_list '}' {
  $$ = iplist_entry_acl ($2);
}
;

/*
 * We allow either "yes", or !0 to mean yes
 */
yes_or_no:C_YES {
  $$ = 1;
}
|C_NO {
  $$ = 0;
}
|NUMBER {
  if ($1 == 1 || $1 == 0) {
    $$ = $1;
  } else {
    err_warn ("invalid number, assuming yes");
    $$ = 1;
  }
}
;

poss_port:			/* Empty */
{
  $$ = htons (0);
}
|C_PORT port {
  $$ = $2;
}
;

/*
 * include "filename";
 */
include:C_INCLUDE QSTRING EOS {
  printf ("include %s\n", $2);
}
;

/*
 * options { ... };
 */
options_block:C_OPTIONS '{' option_lines '}';

option_lines:option_line EOS | option_lines option_line EOS;

option_line:			/* Empty */
|C_DIRECTORY QSTRING		/* done */
{
  SET_STRING (config.directory, $2);
}
|C_NAMED_XFER QSTRING		/* never */
{
  err_warn ("named-xfer option not supported, ignoring");
}
|C_PID_FILE QSTRING		/* done */
{
  SET_STRING (config.pid_file, $2);
}
|C_STATISTICS_FILE QSTRING	/* done */
{
  SET_STRING (config.stats_file, $2);
}
|C_MEMSTATISTICS_FILE QSTRING	/* done */
{
  SET_STRING (config.mem_stats_file, $2);
}
|C_DUMP_FILE QSTRING		/* done */
{
  SET_STRING (config.dump_file, $2);
}
|C_FAKE_IQUERY yes_or_no	/* no plans for support */
{
  err_warn ("fake-iquery option not supported, ignoring");
}
|C_RECURSION yes_or_no		/* done */
{
  config.recursion = $2;
}
|C_FETCH_GLUE yes_or_no		/* no plans for support */
{
  err_warn ("fetch-glue option not supported, ignoring");
}
|C_NOTIFY yes_or_no		/* done */
{
  config.notify = $2;
}
|C_HOST_STATISTICS yes_or_no	/* done */
{
  config.host_statistics = $2;
}
|C_DEALLOCATE_ON_EXIT yes_or_no	/* done */
{
  config.dealloc_on_exit = $2;
}
|C_AUTH_NXDOMAIN yes_or_no	/* done */
{
  config.auth_nxdomain = $2;
}
|C_MULTIPLE_CNAMES yes_or_no	/* done */
{
  config.multiple_cnames = $2;
}
|C_TRANSFERS_IN NUMBER		/* done */
{
  config.transfers_in = $2;
}
|C_TRANSFERS_PER_NS NUMBER	/* done */
{
  config.transfers_per_ns = $2;
}
|C_TRANSFERS_OUT NUMBER		/* done */
{
  config.transfers_out = $2;
}
|C_MAX_TRANSFER_TIME_IN NUMBER	/* done */
{
  config.max_transfer_time_in = $2;
}
|C_TRANSFER_FORMAT transfer_format	/* done */
{
  config.transfer = $2;
}
|C_CHECK_NAMES check_names_type	/* done */
  | C_QUERY_SOURCE query_source	/* done */
  | C_ALLOW_QUERY '{' address_list '}'	/* done */
{
  config.allow_query = $3;
}
|C_ALLOW_TRANSFER '{' address_list '}'	/* done */
{
  config.allow_transfer = $3;
}
|C_TOPOLOGY '{' address_list '}'	/* done */
{
  config.topology = $3;
}
|C_LISTEN_ON poss_port '{' address_list '}' {
  printf ("listen-on %ld\n", $2);
#ifdef DENTS_DEBUG
  print_iplist ($4);
#endif /* DENTS_DEBUG */
}
;

query_source_address:C_ADDRESS wild_addr {
  config.query_source.sin_addr = $2;
}
;

query_source_port:C_PORT wild_port {
  config.query_source.sin_port = htons ($2);
}
;

query_source:query_source_address
  | query_source_address query_source_port
  | query_source_port | query_source_port query_source_address;

zone_block:C_ZONE QSTRING optional_zone_class {
  printf ("zone %s\n", $2);
}
'{' zone_lines '}';

optional_zone_class:		/* Empty */
|STRING {
  printf ("class %s\n", $1);
}
;

/* Fix these numbers */
zone_type:C_HINT {
  $$ = 1;
}
|C_MASTER {
  $$ = 2;
}
|C_SLAVE {
  $$ = 3;
}
|C_STUB {
  $$ = 4;
}
;

zone_lines:zone_line EOS | zone_lines zone_line EOS;

zone_line:			/* Empty */
|C_TYPE zone_type {
  printf ("type %ld\n", $2);
}
|C_FILE QSTRING {
  printf ("file %s\n", $2);
}
|C_MASTERS {
  printf ("adding masters list\n");
}
'{' masters_ipaddr_list '}';

%%int
conf_nc_read (unsigned char *configfn)
{
  int ret;

  if ((ret = lexer_open (configfn)) < 0)
    return ret;

  yyparse ();

  /* FIXME: Where is the definition for this? */
  /* lexer_end_file(); */

  return 0;
}
