/* ctlfac_main.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

#include <string.h>		/* strlen, strncpy */
#include <time.h>		/* getuptime/time() */
#include <stdio.h>		/* ctlfac_handler_thread/fopen() */
#include <orb/orbit.h>		/* ORB mechanisms */

#ifdef HAVE_UNISTD_H
#include <unistd.h>		/* sleep */
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_PTHREAD_H
#include <pthread.h>		/* pthread_create() */
#endif /* HAVE_PTHREAD_H */

#include "config.h"		/* WITH_PTHREAD, etc. */
#include "ctlfac.h"		/* the glue for this whole mess */
#include "ctlfac_impl.h"	/* prototypes for implementation routines */
#include "ctlfac_stats.h"

#include "err.h"

time_t time_of_start;
time_t time_of_reset;
void start_ctlfac_threads (void);
void *ctlfac_handler_thread (void *);
struct ctlfac_stats server_ctlfac_states;

Dents_ctlfac_RecursionType server_recursion_type;
Dents_ctlfac_ResetState server_reset_state;

/* CORBA crap */

Dents_ctlfac ctlfac_client = CORBA_OBJECT_NIL;

PortableServer_ServantBase__epv base_epv;

POA_Dents_ctlfac__epv ctlfac_epv;
POA_Dents_ctlfac__vepv poa_ctlfac_vepv = { &base_epv, &ctlfac_epv };
POA_Dents_ctlfac poa_ctlfac_servant = { NULL, &poa_ctlfac_vepv };

/* code */

void
start_ctlfac_threads (void)
{

#ifdef WITH_PTHREAD
  pthread_t barney;
  int error;

  error = pthread_create (&barney, NULL, ctlfac_handler_thread, NULL);

  if (error)
    err_fatal ("pthread_create() for the ctlfac failed with error %d", error);

#else
  /* Fill in non-thread-based solution here */
#error control facility requires pthreads (for now), but pthread not enabled
#endif /* WITH_PTHREAD */
}

void *
ctlfac_handler_thread (void *dummy)
{
  FILE *iorfd;
  int fargc = 0;
  char *fargv[] = {
    NULL
  };

  PortableServer_ObjectId objid =
    { 0, sizeof ("DentsCtlFac"), "DentsCtlFac" };
  PortableServer_POA poa;

  CORBA_Environment ev;
  char *retval;
  CORBA_ORB orb;

  CORBA_exception_init (&ev);

  orb = CORBA_ORB_init (&fargc, fargv, "orbit-local-orb", &ev);
  if (!orb) {
    err_fatal ("CORBA_ORB_init() returned NULL!");
  }

  /*      Per the CORBA spec, C Language Bindings, I like to
   * initialize my epvs by name at run time, rather than by
   * position at compile time.  This means that as long as
   * I do not change my existing interfaces, I can reorder
   * my .idl file and add new interfaces without breaking
   * anything.  Given that I plan on growing this interface
   * pretty rapidly, this is a nice feature to me.  Of course,
   * leaving unimplemented methods in the final version is very
   * bad, but while under development this is convenient. */

  ctlfac_epv._get_ident = get_ident;
  ctlfac_epv._get_recursion_type = get_recursion_type;
  ctlfac_epv._get_reset_state = get_reset;

  ctlfac_epv.get_uptime = get_uptime;
  ctlfac_epv.get_resettime = get_resettime;
  ctlfac_epv.reset = reset_server;
  ctlfac_epv.get_authans = get_authans;
  ctlfac_epv.get_authnonames = get_authnonames;
  ctlfac_epv.get_authnodata = get_authnodata;
  /* ctlfac_epv. = ; */

  POA_Dents_ctlfac__init (&poa_ctlfac_servant, &ev);

  poa =
    (PortableServer_POA) CORBA_ORB_resolve_initial_references (orb, "RootPOA",
							       &ev);
  if (!poa) {
    err_fatal ("CORBA_ORB_resolve_initial_references() returned NULL!");
  }

  PortableServer_POAManager_activate (PortableServer_POA__get_the_POAManager
				      (poa, &ev), &ev);

  PortableServer_POA_activate_object_with_id (poa, &objid,
					      &poa_ctlfac_servant, &ev);

  ctlfac_client =
    PortableServer_POA_servant_to_reference (poa, &poa_ctlfac_servant, &ev);

  if (!ctlfac_client) {
    fprintf (stderr, "Cannot get objref\n");
    pthread_exit (0);
  }

  retval = CORBA_ORB_object_to_string (orb, ctlfac_client, &ev);

  iorfd = fopen ("/tmp/dents_ctlfac.ior", "w");
  if (!iorfd)
    err_fatal ("Could not open /tmp/dents_ctlfac.ior.");
  fprintf (iorfd, "%s\n", retval);
  fclose (iorfd);

  CORBA_free (retval);

  CORBA_ORB_run (orb, &ev);

  pthread_exit (0);
}
