/* db_rb.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */
/*
 * db_rb.h
 *
 *  Red Black tree implementation
 */
#ifndef _DB_RB_H_
#define _DB_RB_H_

#include <sys/types.h>

typedef struct dbrbnode
{
  struct dbrbnode *left;	/* Left child */
  struct dbrbnode *right;	/* Right child */
  struct dbrbnode *parent;	/* Parent (root if == NULL) */
  guchar color;			/* Color of node (red, black) */
  void *ptr;			/* Value to pass back on find, min, max, etc */
}
dbrbnode;

typedef struct
{
  dbrbnode *root;		/* Pointer to root */
  guint32 numnodes;		/* Number of nodes in the tree */

  int (*compare) ();		/* Function to compare two pieces of data */
}
dbrb;

#define DB_RB_RED	1
#define DB_RB_BLACK	2

/* Leaf place holder */
extern dbrbnode db_rb_nil;

#define DB_RB_NIL (&db_rb_nil)
#define DB_RB_NULL { DB_RB_NIL, DB_RB_NIL, DB_RB_NIL, DB_RB_BLACK, NULL }

/* Prototypes */
gint16 db_rb_insert (dbrb * db, dbrbnode * new, void *ptr);
void db_rb_delete (dbrb * db, dbrbnode * n);
void *db_rb_find (dbrb * db, void *data);
void *db_rb_minimum (dbrb * db);
void *db_rb_maximum (dbrb * db);
void db_rb_create (dbrb * db, int (*compare) ());
void db_rb_walk (dbrbnode * rbnode, void (*callback) (), void *ptr);
void db_rb_print (dbrbnode * rbnode, void (*callback) (), int level);
int db_rb_cmp_uint32 (guint32 * i, guint32 * i1);
int db_rb_cmp_int32 (gint32 * i, gint32 * i1);
int db_rb_cmp_uint16 (guint16 * i, guint16 * i1);
int db_rb_cmp_int16 (gint16 * i, gint16 * i1);
int db_rb_cmp_stri (guchar * s, guchar * s1);
int db_rb_cmp_pstri (guchar ** s, guchar ** s1);

#endif
