/* dents.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifndef DENTS_H
#define DENTS_H

#include "common.h"

#include <stdio.h>
#include <stdarg.h>
#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif /* HAVE_SYSLOG_H */

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif /* HAVE_LIMITS_H */
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>		/* struct timeval (FreeBSD) */
#endif /* HAVE_SYS_TIME_H */

#include <ctype.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>		/* Ironically, for getservbyname */

#include "packet.h"		/* Packet stuff */
#include "sock.h"		/* Socket routines */
#include "link.h"		/* Block allocation/linked list routines */
#include "db.h"			/* Database routines */
#include "conf.h"		/* Config routines */
#include "interface.h"		/* Interface routines */

/* All of the protocols we know of (internal) */
#define PROT_DNS	1
#define PROT_CTL	2

/* Right now, the maximum packet size for any protocol is 64k. Should be */
/*  enough for most things, I hope */
#define MAXPACKETSIZE 64*1024

#ifdef WITH_PTHREAD
/*
 * This structure keeps track of all of the threads we have listening
 */
typedef struct
{
  llentry *ll;		/* Linked List (blocklist) */

  guint16 threadnum;		/* Thead num for printing purposes */
  pthread_t thread;		/* Actual thread structure */

  /* Some protocol specific stats here */
}
receiverrec;
#endif

/*
 * Prototypes
 */
int dents_is_threaded(void);

#endif

