/* dns_answer.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * dns_answer.c
 *
 *  Called whenever we're looking for an answer to a query. Handles
 * authoritative zones, cached answers, recursive queries and any custom
 * functions
 */
#include "common.h"
#include "dns.h"

#if 0
/*
 * dns_ans_acl
 *
 *  Will never return an answer, but can be used to check if the IP we
 * received this query from is allowed to ask us questions.
 *
 * WARNING - this will slow down query processing a little
 */
int
dns_ans_acl (dnspktrec * dnsqpkt, dnsbuildrec * build)
{
  /* FIXME: Actually write the code to check the querying IP address */
  /*  against the info */
  return (DE_NOINFO);
}
#endif

typedef struct
{
  dnsbuildrec *build;
  dnspktrec *dnsqpkt;
  packetrec *pkt;
}
dnsrecursiverec;

/*
 * dns_ans_zt_recursive
 *
 *  Called by db_rb_walk recursively to walk an entire tree
 */
void
dns_ans_zt_recursive (dbnoderec * node, dnsrecursiverec * recurse)
{
  dnsdbrrrec *rr;

  DPRINTF (("dns_ans_zt_recursive: Starting\n"));
  /* NOTE NOTE NOTE  instead of dereferencing the build data structure,
     this routine should call 'dns_build_add()' or 'dns_build_addrr()'
     The dnsbuildrec data structure is no longer published outside of
     dns_build.c. -EZA
   */
#if 0
  recurse->build->lastrr = 1;
  recurse->build->ancount = 1;
#endif

  /* Go through all of the data at this node */
  for (rr = node->data; rr; rr = rr->next) {
    /* SOA is handled differently */
    if (rr->type == DT_SOA)
      continue;

#if 0

    /* Add it and send */
    recurse->build->rrptr[0] = rr;
    dns_buildpacket (recurse->pkt, &recurse->dnsqpkt->query, recurse->build,
		     recurse->dnsqpkt->id, DE_NOERROR,
		     recurse->dnsqpkt->flags, 1,
		     (recurse->dnsqpkt->qru.query.zone->
		      flags & DNS_ZONE_AUTHORITATIVE));
    dns_sendpacket (recurse->pkt->sock, recurse->pkt, &recurse->dnsqpkt->ip,
		    recurse->dnsqpkt->port);
    /*
       recurse->pkt->sock->send(recurse->pkt->sock, recurse->pkt,
       &recurse->dnsqpkt->ip, recurse->dnsqpkt->port);
     */
#endif
  }

  db_rb_walk (node->children.root, dns_ans_zt_recursive, recurse);
}

/*
 * dns_ans_zone_transfer
 *
 *  Handles doing zone transfers
 */
void
dns_ans_zone_transfer (dnspktrec * dnsqpkt, dnsbuildrec * build,
		       packetrec * pkt)
{
  dnsdbrrrec *rr, *soarr = NULL;
  dbnoderec *node;
  dnsrecursiverec recurse;

  recurse.build = dns_build_alloc ();

  /* FIXME: Stop compile warnings */
  node = NULL;
  recurse.build = (dnsbuildrec *) node;
#if 0
  /* Let's see if we can find any info on this domain name */
  if ((node = dns_db_find (&dns.db, dnsqpkt->query.name)) == NULL)
    return;
#endif

  /* Loop through and find all of the RR's for this requested type */
  rr = NULL;
  while (node && ((rr = dns_db_rr_findmatch (node, rr, dnsqpkt->query.class,
					     DT_SOA)) != NULL)) {
    if (rr->type != DT_SOA)
      continue;

    soarr = rr;
    break;
  }

  if (!soarr) {
    err_warn ("dns_ans_zone_transfer: no SOA found for %s",
	      dnsqpkt->query.name);
    return;
  }
#if 0
  /* Start off with the SOA */
  build->lastrr = 0;
  build->rrptr[build->lastrr++] = soarr;
  build->ancount = 1;

  dns_buildpacket (pkt, &dnsqpkt->query, build, dnsqpkt->id, DE_NOERROR,
		   dnsqpkt->flags, 1, (dnsqpkt->qru.query.zone->flags &
				       DNS_ZONE_AUTHORITATIVE));
  dns_sendpacket (pkt->sock, pkt, &dnsqpkt->ip, dnsqpkt->port);
  /*
     pkt->sock->send(pkt->sock, pkt, &dnsqpkt->ip, dnsqpkt->port);
   */

  /* FIXME: Do multiple RR's per packet (user configurable) */
  /* Default it to 1 per packet */
  build->lastrr = 1;
  build->ancount = 1;

  /* Go through all of the data at this node */
  for (rr = node->data; rr; rr = rr->next) {
    /* SOA is handled differently */
    if (rr->type == DT_SOA)
      continue;

    /* Add it and send */
    build->rrptr[0] = rr;
    dns_buildpacket (pkt, &dnsqpkt->query, build, dnsqpkt->id, DE_NOERROR,
		     dnsqpkt->flags, 1, (dnsqpkt->qru.query.zone->flags &
					 DNS_ZONE_AUTHORITATIVE));
    dns_sendpacket (pkt->sock, pkt, &dnsqpkt->ip, dnsqpkt->port);
    /*
       pkt->sock->send(pkt->sock, pkt, &dnsqpkt->ip, dnsqpkt->port);
     */
  }

  recurse.build = build;
  recurse.pkt = pkt;
  recurse.dnsqpkt = dnsqpkt;

  /* Walk this way */
  db_rb_walk (node->children.root, dns_ans_zt_recursive, &recurse);

  /* Finish off with the SOA */
  build->lastrr = 0;
  build->rrptr[build->lastrr++] = soarr;
  build->ancount = 1;

  dns_buildpacket (pkt, &dnsqpkt->query, build, dnsqpkt->id, DE_NOERROR,
		   dnsqpkt->flags, 1, (dnsqpkt->qru.query.zone->flags &
				       DNS_ZONE_AUTHORITATIVE));
  dns_sendpacket (pkt->sock, pkt, &dnsqpkt->ip, dnsqpkt->port);

  dns_build_delete (recurse->build);
  /*
     pkt->sock->send(pkt->sock, pkt, &dnsqpkt->ip, dnsqpkt->port);
   */
#endif
}

typedef int (*chain_func_t) ();
/*
 * dns_ans_dochains
 *
 *  Roll's through all of the functions associated with a zone. Called for
 * new queries and for each part of a recursive query
 */
int
dns_ans_dochains (dnspktrec * dnsqpkt, dnsbuildrec * build, packetrec * pkt)
{
  dbnoderec *node;
  dnschainrec *chain;
  int rc;
  chain_func_t *func;

  /* Find the zone it's in */
  if ((node = dns_db_findbest (&dns.zonedb, dnsqpkt->query.name)) == NULL) {
    /* No zone that covers this. This usually means we have no cache */
    /*  file. This may be what they wanted, return SERVFAIL */
    err_fatal ("dns_ans_dochains: couldn't find best for %s",
	       dnsqpkt->query.name);

    return (DE_SERVFAIL);
  }

  DPRINTF (("dns_ans_dochains: found zone at %p\n", node));

  dnsqpkt->qru.query.zone = node->data;

  /* Go through all of the functions finding info */
  rc = DE_NOINFO;

  /* Go through all of the modules in the chain */
  for (chain = link_first (&dnsqpkt->qru.query.zone->lookup);
       chain && (rc == DE_NOINFO); chain = link_next (chain, chain->ll)) {
    /* And go through all of the functions in the list */
    for (func = chain->module->answer; *func && (rc == DE_NOINFO); func++) {
      rc = (*func) (dnsqpkt, build, pkt);
      DPRINTF (("dns_ans_dochains: func returned %d\n", rc));
    }
  }

  return (rc);
}

#if 0
/*
 * dns_ans_callback
 *
 *  Callback for the OSQ code. whenever we get a reply back, this
 * will be called
 */
int
dns_ans_callback (dnspktrec * dnsqpkt, dnsbuildrec * build, packetrec * pkt)
{
  int rc;

  rc = dns_ans_dochains (dnsqpkt, build, pkt);

  if (rc == DE_WAITFORMOREINFO)
    return (0);

  dns_buildpacket (pkt, &dnsqpkt->query, build, dnsqpkt->qru.query.zone,
		   dnsqpkt->id, rc, dnsqpkt->flags, 1);

  dns_sendpacket (dnsqpkt->socket, pkt, dnsqpkt->ip, dnsqpkt->port);
  /*
     pkt->sock->send(dnsqpkt->socket, pkt, &dnsqpkt->ip, dnsqpkt->port);
   */

  return (1);
}
#endif
