/* dns_qr_resp.c - Function to handle received DNS response-packets.

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"
#include "dns.h"

#include <arpa/inet.h>		/* inet_ntoa */

/* Called when we get a DNS packet which is a reply.  Will search the
list of outstanding queries we've sent out and then match it up to the
correct thread to handle the information returned. */

void
dns_resp_handlepkt (dnspktrec * dnsrpkt, packetrec * pkt, guchar * cp)
{
  dnsosqueryrec *osq;

  /* Try to find the osq by id */
  if ((osq = dns_osq_findid (dnsrpkt->id)) == NULL) {
    /* Hmm, didn't find it. Could be a duplicate */
    err_warn ("dns_resp_handlepkt: couldn't find osq for response!");
    return;
  }

  DPRINTF (("dns_resp_handlepkt: found osq @ %p to %s:%d\n", osq,
	    inet_ntoa (osq->toip), osq->toport));
  DPRINTF (("dns_resp_handlepkt: osq name: %s, class: %d, type: %d\n",
	    osq->query.name, osq->query.class, osq->query.type));

  /* Callback */
  osq->callback (osq, pkt, cp, dnsrpkt);
}
