/* dns_sock.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */
/*
 * dns_sock.h
 *
 *  Socket interface as it relates to DNS
 */
#ifndef DNS_SOCK_H
#define DNS_SOCK_H

#include "sock.h"

/* Private structure for DNS TCP sockets */
typedef struct
{
  llentry ll;			/* All of the DNS TCP connections */

  socketrec *socket;		/* The socket structure associated with us */

  struct in_addr ip;		/* If !LISTENING, rmt IP, else local IP */
  guint16 port;			/* If !LISTENING, rmt port, else local port */

  guint16 packetsize;		/* Size of the actual packet we read */
  /*  or are writing */
  guint16 packetread;		/* Amount that we've read so far */

  packetrec packet;		/* Since we need to buffer between */
  /*  reads anyway */
  guint32 buffersize;		/* Size of the TCP buffer */
  guint16 bufferstart;		/* Where it starts in the first block */
  guint16 bufferend;		/* Where it ends in the last block */
  linkedlist buffer;		/* We buffer TCP streams */
}
dnstcprec;

/* Private structure for DNS UDP sockets */
typedef struct
{
  llentry ll;			/* All of the DNS UDP sockets */

  socketrec *socket;		/* The socket structure associated with us */

  struct in_addr ip;		/* Local IP */
  guint16 port;			/* Local port */
}
dnsudprec;

/*
 * Prototypes
 */

/* dns_sock_tcp.c */
void dns_sock_tcp_accept (struct socketrec *sock, struct packetrec *pkt);
void dns_sock_tcp_readsize (struct socketrec *sock, struct packetrec *pkt);
void dns_sock_tcp_readpacket (struct socketrec *sock, struct packetrec *pkt);
void dns_sock_tcp_send (struct socketrec *sock, guchar * buf, int length,
			struct in_addr *ip, guint16 port);
void dns_sock_tcp_sendbuf (struct socketrec *sock);
void dns_sock_tcp_pullzone (struct socketrec *sock, struct packetrec *pkt);
void dns_sock_tcp_sendaxfr (socketrec * sock);
socketrec *dns_sock_tcp_alloc (void);
void dns_sock_tcp_free (struct socketrec *sock);
void dns_sock_tcp_bind (void);

/* dns_sock_udp.c */
void dns_sock_udp_send (struct socketrec *sock, guchar * buf, int length,
			struct in_addr *ip, guint16 port);
void dns_sock_udp_read (struct socketrec *sock, struct packetrec *pkt);
socketrec *dns_sock_udp_alloc (void);
void dns_sock_udp_free (struct socketrec *sock);
void dns_sock_udp_bind (void);

#if 0
/* Socket Operations */
socketops dnstcpops = { dns_sock_tcp_delete, dns_sock_tcp_send };
socketops dnsudpops = { dns_sock_udp_delete, dns_sock_udp_send };
#endif

#endif
