/* dns_zone.h - various stuff dealing with zones and the like.

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifndef DNS_ZONE_H
#define DNS_ZONE_H

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif /* HAVE_LIMITS_H */

/* This describes what kind of authority over this zone we have. */
#define DNS_ZONE_PRIMARY	1
#define DNS_ZONE_SECONDARY	2

/* If we're the master for a zone */
#define DNS_ZONE_MASTER		0x0001
#define DNS_ZONE_CACHE		0x0002
#define DNS_ZONE_AUTHORITATIVE	0x0004

enum
{
  DNS_ZONE_LOADED,
  DNS_ZONE_NOTLOADED,
  DNS_ZONE_ERRORLOADING
};

/* struct prototyping */
struct dnsmodulerec;

/* Strcture to track each zone we have loaded */
typedef struct dnszonerec
{
  llentry ll;			/* Linked List (blocklist) */

  dbnoderec *dbnode;		/* Node in the zone db */
  guint16 flags;		/* Flags for this zone */
  int status;			/* Status for the zone */

  guchar zonename[DNS_MAXNAMELENGTH + 1];	/* Name */
#ifndef PATH_MAX
#define PATH_MAX 100
#warning Had to guess PATH_MAX for some reason. (?)
#endif
  guchar filename[PATH_MAX + 1];	/* File name to zone file */

  struct in_addr master;	/* IP of master, undefined if not */

  linkedlist lookup;		/* Modules */

  struct dnsmodulerec *loadmodule;	/* Module responsible for (re)loading */
  void *mdata;			/* Module data - for use by the module author */

  int argc;			/* # of arguments in the list below     */
  char **argv;			/* extra module parameters for this zone */

}
dnszonerec;

/* Strcture to track each zone we have loaded */
typedef struct
{
  llentry ll;			/* Linked List (blocklist) */

  guchar zonename[DNS_MAXNAMELENGTH + 1];	/* Name of the zone */
  guchar filename[PATH_MAX + 1];	/* File name to zone file */
}
dnshintrec;

/* dns_zone.c */
void dns_zone_addmodule (dnszonerec * zone, struct dnsmodulerec *module);
dnszonerec *dns_zone_create (guchar * zonename);
void dns_zone_setloadmod (dnszonerec * zone, struct dnsmodulerec *module);
void dns_zone_setslaveip (dnszonerec * zone, struct in_addr ip);
void dns_zone_setfilename (dnszonerec * zone, guchar * fnfmt, ...);
void dns_zone_loadall (void);
void dns_zone_scheduletransfer (dnszonerec * zone);
void dns_db_addzone (dbrootrec * db, guchar * domainname, dnszonerec * zone);
void dns_zone_cleanup (dnszonerec * zone);

#endif
