/* module.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */
/*
 * module.h
 *
 *  Generic interface some common module code
 */
#ifndef MODULE_H
#define MODULE_H

#include "common.h"

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif /* HAVE_LIMITS_H */
#include "link.h"

/* Symbol information about static modules */
typedef struct
{
  guchar *name;
  void *ptr;
}
modulesymbolrec;

/* Maximum length of module name */
#define MODULE_MAXNAMELENGTH	255

typedef enum
{
  NOTLOADED = 0,		/* Not loaded */
  LOADED,			/* Loaded, not initialized */
  ACTIVE,			/* Loaded, initialized and active */
  INACTIVE			/* Loaded, initialized and inactive */
}
module_state_t;

typedef enum
{
  STATIC,			/* Compiled into the server */
  DYNAMIC			/* Dynamically loaded at runtime */
}
module_type_t;

/* Module */
typedef struct modulerec
{
  llentry ll;			/* Linked List (blocklist) */

  module_state_t state;		/* Current state of the module */
  module_type_t type;		/* Type of module (dynamic, static) */

  guchar name[MODULE_MAXNAMELENGTH + 1];	/* Name from config */
  guchar *modname;		/* Name from module */
  guchar pathname[PATH_MAX + 1];	/* Path */

  void *handle;			/* Handle returned (dynamic) */

  int refcount;			/* Reference count */

  /* Version of dents the module was compiled against. */
  char *compiled_version;

  /* Optional functions */
  int (*init) (void);		/* Function to initialize */
  void (*destroy) (void);	/* Function to destroy */

  /* Optional variables */
  char *version;
}
modulerec;

/* Prototypes */
/* module.c */
int module_populate (modulerec * module);
modulerec *module_alloc (void);
void module_free (modulerec * module);
modulerec *module_open (guchar * filename);
int module_close (modulerec * module);
void *module_sym (modulerec * module, guchar * symname);

/* module_static.c */
void *module_st_sym (modulerec * module, guchar * symname);
void module_st_init (void);

/* module_dynamic.c */
int module_dyn_path_add (guchar * pathname);
int module_dyn_path_del (guchar * pathname);
void *module_dyn_sym (modulerec * module, guchar * symname);
modulerec *module_dyn_open (guchar * modname);
int module_dyn_close (modulerec * module);

/* module_{dl}.c */
void *module_i_open (guchar * filename);
int module_i_close (void *handle);
void *module_i_sym (void *handle, guchar * symname);

#endif /* !MODULE_H */
