/* module_dl.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * module_dl.c
 *
 *  Code to handle libdl style dynamically loaded libraries
 */
#include "common.h"
#include "module.h"
#include <dlfcn.h>

/* Some systems lack RTLD_GLOBAL so we give them a harmless default */
#ifndef RTLD_GLOBAL
#define RTLD_GLOBAL	0
#endif

/*
 * module_i_open
 *
 *  Opens a module named by filename and returns an "opaque" reference to
 * it
 */
void *
module_i_open (guchar * filename)
{
  void *handle;

  /* We want all symbols resolved now since doing it at runtime can */
  /*  be dangerous */
  DPRINTF (("module_i_open: opening %s\n", filename));
  if ((handle = dlopen (filename, RTLD_NOW | RTLD_GLOBAL)) == NULL) {
    err_warn ("(dl)module_i_open: dlopen(%s): %s", filename, dlerror ());
    return (NULL);
  }

  return (handle);
}

/*
 * module_i_close
 *
 *  Closes an opened module
 */
int
module_i_close (void *handle)
{
  if (dlclose (handle) != 0) {
    /* failure */
    err_warn ("(dl)module_i_close: %s", dlerror ());
    return (-1);
  }

  return (0);
}

/*
 * module_i_sym
 *
 *  Tries to determine the address of a symbol in a loaded library
 */
void *
module_i_sym (void *handle, guchar * symname)
{
  /* We don't warn here since the higher layers will for us */
  return (dlsym (handle, symname));
}
