/* module_static.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * module_static.c
 *
 *  Handles statically linked modules
 */
#include "module.h"
#include "common.h"

#include "module_static.h"

/*
 * module_st_sym
 *
 *  Returns a pointer to a symbol
 */
void *
module_st_sym (modulerec * module, guchar * symname)
{
  modulesymbolrec *sym;

  for (sym = module->handle; sym->name != NULL; sym++) {
    if (strcmp (sym->name, symname) == 0)
      return (sym->ptr);
  }

  return (NULL);
}

/*
 * module_st_open
 *
 *  Open's a statically linked module
 */
int
module_st_open (guchar * modname, modulesymbolrec * syms)
{
  modulerec *module;

  /* Allocate a structure */
  if ((module = module_alloc ()) == NULL) {
    err_warn ("module_st_alloc: couldn't allocate memory for module");
    return (-1);
  }

  module->refcount = 0;		/* Start at 0 */

  /* Keep the symbol information */
  module->handle = syms;

  /* Save the module name from the configure script */
  strncpyzt (module->name, modname, MODULE_MAXNAMELENGTH);

  module->type = STATIC;

  /* No path, so skip it */
  module->pathname[0] = 0;

  /* Try populating the symbols */
  if (module_populate (module) < 0) {
    err_warn ("module_st_open: module %s was missing symbols", modname);

    module_free (module);
    return (-1);
  }

  return (0);
}

/*
 * module_st_init
 *
 *  Initializes module structs for all static modules
 */
void
module_st_init (void)
{
  staticmodulerec *stmod;

  /* Go through all of the modules */
  for (stmod = modules_static; stmod->name != NULL; stmod++) {
    DPRINTF (("module_st_init: setting up %s\n", stmod->name));

    if (module_st_open (stmod->name, stmod->syms) < 0) {
      err_warn ("module_st_init: couldn't open static module %s @ %p",
		stmod->name, stmod);
    }
  }
}
