/* packet.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * packet.c
 *
 *  Handles the generic packet data type
 *
 * FIXME: We should probably cache buf's in the future
 */
#include "common.h"
#include "packet.h"

linkedlist packets = LINK_INITIALIZER (packetrec, ll);
blocklist packet_pool = BLOCK_INITIALIZER (packetrec, 1);

/*
 * packet_resizepfbuf
 *
 *  Resize's the packet buffer by a fixed amount
 */
int
packet_resizepbuf (packetrec * pkt)
{
  void *ptr;

  if ((ptr = realloc (pkt->packetbuf, pkt->pbufsize + PACKET_BUF_INCSIZE))
      == NULL)
    return (-1);

  pkt->packetbuf = ptr;
  pkt->pbufsize += PACKET_BUF_INCSIZE;

  return (1);
}

/*
 * packet_resizetbuf
 *
 *  Resize's the packet buffer by a fixed amount
 */
int
packet_resizetbuf (packetrec * pkt)
{
  void *ptr;

  if ((ptr = realloc (pkt->tempbuf, pkt->tbufsize + PACKET_TMP_INCSIZE))
      == NULL)
    return (-1);

  pkt->tempbuf = ptr;
  pkt->tbufsize += PACKET_TMP_INCSIZE;

  return (1);
}

/*
 * packet_alloc
 *
 *  Allocate's a packet
 */
packetrec *
packet_alloc (void)
{
  packetrec *pkt;

  pkt = block_allocandaddtolink (&packet_pool, &packets);

  pkt->pbufsize = PACKET_BUF_DEFSIZE;
  pkt->tbufsize = PACKET_TMP_DEFSIZE;

  if ((pkt->packetbuf = malloc (pkt->pbufsize)) == NULL)
    err_fatal ("packet_alloc: malloc: %m");

  if ((pkt->tempbuf = malloc (pkt->tbufsize)) == NULL)
    err_fatal ("packet_alloc: malloc: %m");

  return (pkt);
}

/*
 * packet_free
 *
 *  Free's a packet
 */
void
packet_free (packetrec * pkt)
{
  free (pkt->packetbuf);
  free (pkt->tempbuf);

  block_delete (&packet_pool, &pkt->ll);
}
