/* packet.h - handles the generic packet data type.

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifndef PACKET_H
#define PACKET_H

#include <netinet/in.h>
#include "link.h"

/* Default sizes, they should grow if needed */
#define PACKET_BUF_DEFSIZE	4096
#define PACKET_TMP_DEFSIZE	4096

#define PACKET_BUF_INCSIZE	1024
#define PACKET_TMP_INCSIZE	1024

/* Data type prototyping */
struct socketrec;

/*
 * All packets get thrown into this structure. This is the main method of
 *  passing the packet through the different functions
 */
typedef struct packetrec
{
  llentry ll;			/* Linked List (blocklist) */

  struct in_addr ip;		/* Always the IP the packet came from */
  guint16 port;			/*  and the port */
  struct socketrec *sock;	/* Which socket the packet went to */

  guint16 packetsize;		/* Actual size of the read packet */

  /* Buffer for the actual packet */
  guint16 pbufsize;		/* Maximum size of the packet buffer */
  guchar *packetbuf;		/* Actual buffer */

  /* Temp space (unpacking RR's, etc) */
  guint16 tbufsize;		/* Maximum size of the temp buffer */
  guchar *tempbuf;		/* Actual buffer */
}
packetrec;

/*
 * Prototypes
 */

/* packet.c */
int packet_resizepbuf (packetrec * pkt);
int packet_resizetbuf (packetrec * pkt);
packetrec *packet_alloc (void);
void packet_free (packetrec * pkt);

#endif /* PACKET_H */
