/* sock.h - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */
/*
 * sock.h
 *
 *  Various generic socket handling routines. Provides an easy to use API
 * for dealing with TCP, UDP and Unix Domain Sockets (UDS)
 */
#ifndef SOCK_H
#define SOCK_H

#include <sys/socket.h>		/* FreeBSD support */

#include "interface.h"
#include "protocol.h"
#include "link.h"

/* Mmm, prototyping structures */
struct socketrec;
struct packetrec;

/* TCP and UDP specific stuff */
#include "sock_tcp.h"
#include "sock_udp.h"

#if 0
/* Socket flags */
#define SOCK_WRITENOTIFY	0x0001
#endif

/* Different socket types */
enum
{
  SOCKET_UDP,
  SOCKET_TCP,
  SOCKET_UNIX
};

enum
{
  SOCK_STATE_LISTENING,
  SOCK_STATE_READ,
  SOCK_STATE_READSIZE,
  SOCK_STATE_READPACKET,
  SOCK_STATE_READAXFR
};

#if 0
/* Various socket operations */
typedef struct socketops
{
  /* Delete the socket */
  void (*delete) (struct socketrec *);
  /* Send arbitrary information to the socket */
  void (*send) (struct socketrec *, guchar *, int, struct in_addr, guint16);
}
socketops;
#endif

/*
 * Data type used to track all open sockets. Also contains information
 *  private to the type of socket (TCP, UDP, UDS) and protocol (DNS, CTL)
 */
typedef struct socketrec
{
  llentry ll;			/* Linked list of all sockets */

  int fd;			/* File Descriptor */
  interfacerec *interface;	/* Bound Interface */
#if 0
  int flags;			/* Flags */
#endif

  llentry lt;			/* Linked list of sockets for this type */
  int type;			/* SOCKET_UDP, SOCKET_TCP, SOCKET_UNIX, etc */
  int state;			/* Current state of socket */

  union
  {				/* Type Private stuff */
    tcppriv tcp;		/* TCP */
    udppriv udp;		/* UDP */
  }
  typepriv;

  llentry lp;			/* Linked list of sockets for this protocol */
  protocolops *pops;		/* Protocol operations */

  void *priv;			/* Private data for this type/protocol */

  void (*readaction) (struct socketrec *, struct packetrec *);
  /* Callback when socket is ready to read */
  /*  Usually a type/protocol function */
  void (*writeaction) (struct socketrec *);
  /* Callback when socket is ready to write */
  /*  Usually a type function */
}
socketrec;

/* Shortcuts */
#define tp_tcp typepriv.tcp
#define tp_udp typepriv.udp

/* Prototypes */
/* sock.c */
int sock_open (int proto, struct in_addr ip, guint16 port, int nonblock);
void sock_setnonblock (int fd);
socketrec *sock_alloc (void);
void sock_free (socketrec * sock);

#endif
