/* sock_tcp.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * sock_tcp.c
 *
 *  Sockets for TCP
 */
#include "common.h"
#include "sock.h"

/* List of all TCP sockets */
linkedlist tcpsockets = LINK_INITIALIZER (socketrec, lt);

/* Pool of TCP buffers */
blocklist tcpbuffer_pool = BLOCK_INITIALIZER (tcpbuffer, 5);

#if 0
socketops tcpsocketops = {
  sock_tcp_free,
  sock_tcp_write
};
#endif

/*
 * sock_tcp_sendbuf
 *
 *  Called when we can write to the socket
 * FIXME: Use writev for performance in some circumstances
 */
int
sock_tcp_sendbuf (socketrec * sock)
{
  int numwrite;
  int numwritten;

  /* Write as much as we can to the socket */
  do {
    /* How much can we write from this block? */
    if (sock->tp_tcp.bufferstart + sock->tp_tcp.buffersize > TCP_BUFFER_SIZE)
      numwrite = TCP_BUFFER_SIZE - sock->tp_tcp.bufferstart;
    else
      numwrite = sock->tp_tcp.buffersize;

    /* Write to the socket */
    if ((numwritten = write (sock->fd, ((tcpbuffer *)
					link_first (&sock->tp_tcp.
						    buffer))->buffer +
			     sock->tp_tcp.bufferstart, numwrite)) < 0) {
      if (errno == EAGAIN)	/* Can't write anymore, ok */
	numwritten = 0;
      else			/* Bad */
	return (numwritten);
    }

    /* Update the pending writes with how much we got rid of */
    /*  this write */
    sock->tp_tcp.buffersize -= numwritten;

    /* Did we finish off this block? */
    if (numwritten == numwrite) {
      sock->tp_tcp.bufferstart = 0;
      block_delete (&tcpbuffer_pool,
		    &((tcpbuffer *) link_first (&sock->tp_tcp.buffer))->ll);
    }
  } while (numwritten == numwrite);

  /* It's atleast a little bit accurate, definately not < 0 */
  return (numwritten);
}

/*    
 * sock_tcp_write
 *
 *  Called whenever we want to write to a socket. If we can write to it
 * we will, otherwise, we'll buffer up
 */
int
sock_tcp_write (socketrec * sock, guchar * buf, int bufsize)
{
  int numcopy;

  /* If the buffer is empty, try writing to the socket immediately */
  /*  it's probably empty */
  if (!sock->tp_tcp.buffersize) {
    int numwritten;

    /* See if we can write all we have to the socket */
    if ((numwritten = write (sock->fd, buf, bufsize)) < 0) {
      if (errno == EAGAIN)
	numwritten = 0;
      else
	return (numwritten);
    }

    /* Cool, it all wrote at once so no buffering */
    if (numwritten == bufsize)
      return (numwritten);

    /* Update the info to reflect the amount written */
    buf += numwritten;
    bufsize -= numwritten;
  }

  /* Now copy it over */
  do {
    /* Figure out how much we can copy */
    numcopy = TCP_BUFFER_SIZE - sock->tp_tcp.bufferend;

    /* Copy it over */
    memcpy (((tcpbuffer *) sock->tp_tcp.buffer.end)->buffer +
	    sock->tp_tcp.bufferend, buf, numcopy);

    /* Update the counters */
    sock->tp_tcp.bufferend += numcopy;
    bufsize -= numcopy;
    buf += numcopy;
    sock->tp_tcp.buffersize += numcopy;

    /* If we still have some buffer left, let's allocate some more memory */
    if (bufsize) {
      block_allocandaddtolink (&tcpbuffer_pool, &sock->tp_tcp.buffer);
      sock->tp_tcp.bufferend = 0;
    }
  } while (bufsize);

  /* We trash bufisize */
  return (0);
}

/*
 * sock_tcp_alloc
 *
 *  Allocate's a TCP socket
 */
socketrec *
sock_tcp_alloc (void)
{
  socketrec *sock;

  /* Allocate a normal socket */
  sock = sock_alloc ();

  /* Add it into the linked list */
  link_add (&tcpsockets, &sock->lt);

  /* Setup the necessary fields */
  sock->type = SOCKET_TCP;

  /* Reset the TCP private stuff */
  sock->tp_tcp.buffersize = sock->tp_tcp.bufferstart =
    sock->tp_tcp.bufferend = 0;

  return (sock);
}

/*
 * sock_tcp_free
 *
 *  Free's a previously allocated TCP socket
 */
void
sock_tcp_free (socketrec * sock)
{
  /* Remove it from the linked list */
  link_delete (&tcpsockets, &sock->lt);

#if 0
  sock_free (sock);
#endif
}
