/* sock_udp.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * sock_udp.c
 *
 *  UDP socket stuff
 */
#include "common.h"
#include "sock.h"

#include <string.h>		/* memset */

/* UDP sockets */
linkedlist udpsockets = LINK_INITIALIZER (socketrec, lt);

#if 0
socketops udpsocketops = {
  sock_udp_free,
  sock_udp_write
};
#endif

/*
 * sock_udp_write
 *
 *  Write's to a UDP socket
 */
int
sock_udp_write (struct socketrec *sock, guchar * buf, int length,
		struct in_addr *ip, guint16 port)
{
  struct sockaddr_in sa;

  /* Zero it out */
  memset ((void *) &sa, 0, sizeof (sa));

  /* Fill in the information */
  sa.sin_addr.s_addr = ip->s_addr;
  sa.sin_port = htons (port);
  sa.sin_family = AF_INET;

  return (sendto (sock->fd, buf, length, 0, (struct sockaddr *) &sa,
		  sizeof (sa)));
}

/*
 * sock_udp_alloc
 *
 *  Allocate's a UDP socket
 */
socketrec *
sock_udp_alloc (void)
{
  socketrec *sock;

  /* Allocate the socket */
  sock = sock_alloc ();

  /* Add it to the linked list */
  link_add (&udpsockets, &sock->lt);

  /* Setup the necessary fields */
  sock->type = SOCKET_UDP;

  return (sock);
}

/*
 * sock_udp_free
 *
 *  Free's UDP specific data
 */
void
sock_udp_free (socketrec * sock)
{
  /* Nothing */
}
