/* -*- C -*-
 * conf_nc_lexer.l
 *
 * LEX file for lexical analysis of output of arg_list_write
 *
 */

%{
/*
 * C Code included at top of generated scanner
 */
#include <string.h>		/* strdup, etc */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>		/* inet_ntoa */
#include "err.h"		/* error handling routines               */
#include "arg_list_api.h"	/* ARG_HND defines */
#include "al_parse_defs.h"	/* pre-processor definitions to avoid globals*/
#include "arg_list_parse.h"	/* generated token definitions from yacc */

int yy_al_lineno;

#define YY_NO_UNPUT	/* We don't need yyunput */

/*
 * Regular Expressions that define tokens
 */


%}

ws		[ \t]+
hcomment	#.*
cppcomment	\/\/.*
ccomment	\/\*[^\*]*\*\/
number          [+-]{0,1}[0-9]+
qstring		\'[^\']*\'
string		[a-zA-z_][a-zA-z0-9]*
nl		\n

%%


	/* We do blank lines, hash comments and C++ style comments here */
{ws}		;
{hcomment}	;
{cppcomment}	;

\=		return '=';
\:		return ':';
\{		return '{';
\}		return '}';
\[		return '[';
\]		return ']';
\,              return ',';

	/* Special types */
{qstring}	{ yylval.string = strdup(yytext + 1);
		  if (yylval.string[yyleng - 2] != '\'')
		    err_warn("Unterminated character string at line %d", 
			     yy_al_lineno);
		   else
		    yylval.string[yyleng - 2] = 0;

		  return(AL_STRING);
		}

{string}	{ yylval.string = strdup(yytext);
		  return(AL_STRING);
		}

{number}        { 
                if (sscanf (yytext,"%ld", &yylval.num) !=1) {
		    err_warn("Couldn't parse %s as a number at line %d", 
			     yytext, yy_al_lineno);
                  }
		  return (AL_NUMBER);
                }
		/* Count the lines */

{nl}		;	/* it seems that flex is already keeping count */


%%

/*
 * C Code included at bottom of generated scanner file
 */

/* If we define this here, we can avoid using the fl library */
/* Could have used %option noyywrap */
#ifndef yy_al_wrap
int yy_al_wrap(void)
{
  return(1);
}
#endif


/*
 * yy_al_scanner  -  Use this function to scan a string instead of a FILE * 
 */
void yy_al_scanner (char * string)
{
  yy_delete_buffer (YY_CURRENT_BUFFER);
  yy_switch_to_buffer (yy_scan_string(string));
  BEGIN(INITIAL);
}
