/* arg_list_parse.y - YACC file for al_hnd_write()-output.  */
%{
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>

#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "err.h"
#include "arg_list_api.h"
#include "al_parse_defs.h"

/* Extenal function prototypes */
int yylex(void);

#define YYDEBUG 1
#define YYERROR_VERBOSE	1

/* External variables defined in arg_list_lexer.l */
extern int yy_al_lineno;

/* Global variables exported from this file */
int        yy_al_debug;
ARG_HND    global_al_parsed_hnd;
char      *global_al_parsed_name;

/* Local function prototypes */
static void yyerror(char *, ...);

%}

/* each $ arg is a union (a sub-member of this union ) */
%union {
	char *string;
	long num;
        ARG_HND  ah;
        ARG_LIST al;
}

%token <string> AL_STRING
%token <num>    AL_NUMBER
%token <error>  AL_ERROR
%type <ah>  named_hash_values
%type <ah>  named_hash_value_list
%type <al>  arg_list
%type <al>  arg_list_values

%%

start:	named_hash
	;

named_hash: AL_STRING '=' named_hash_value_list
        {
	  /* Assign some global variable here */
	  global_al_parsed_hnd = $3;
	  global_al_parsed_name = $1;
	}
	;

named_hash_value_list : '{' named_hash_values '}'
	{
	  $$=$2;
	}
	| '{' '}'
	{
	  if (!($$ = al_hnd_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't make a new ARG_HND ");
	  }
	}
	
named_hash_values :  named_hash_values ',' AL_STRING ':'  AL_STRING 
        {
	  if (al_hnd_add_string ($1, $3, $5)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add string to ARG_HND");
	  }
	  free ($3);
	  free ($5);
	  $$ = $1;
	}
        | AL_STRING ':'  AL_STRING 
        {
	  if (!($$ = al_hnd_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't make a new ARG_HND ");
	  }
	  if (al_hnd_add_string ($$, $1, $3)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add string to ARG_HND");
	  }
	  free ($1);
	  free ($3);
	}
        | named_hash_values ',' AL_STRING ':'  AL_NUMBER 
        {
	  if (al_hnd_add_int ($1, $3, $5)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add number to ARG_HND");
	  }
	  free ($3);
	  $$ = $1;
        }
        | AL_STRING ':'  AL_NUMBER 
        {
	  if (!($$ = al_hnd_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't make a new ARG_HND ");
	  }
	  if (al_hnd_add_int ($$, $1, $3)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add number to ARG_HND");
	  }
	  free ($1);
        }
        | named_hash_values ',' AL_STRING ':'  arg_list 
        {
	  if (al_hnd_add_list ($1, $3, $5)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add list to ARG_HND");
	  }
	  free ($3);
	  $$ = $1;	  
        }
        | AL_STRING ':'  arg_list 
        {
	  if (!($$ = al_hnd_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't make a new ARG_HND ");
	  }

	  if (al_hnd_add_list ($$, $1, $3)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add list to ARG_HND");
	  }
	  free ($1);
        }
        | named_hash_values ',' AL_STRING ':'  named_hash_value_list 
        {
	  if (al_hnd_add_hnd ($1, $3, $5)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add hnd to ARG_HND");
	  }
	  free ($3);
	  $$ = $1;	  
        }
        | AL_STRING ':' named_hash_value_list 
        {
	  if (!($$ = al_hnd_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't make a new ARG_HND ");
	  }

	  if (al_hnd_add_hnd ($$, $1, $3)  != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add hnd to ARG_HND");
	  }
	  free ($1);
        }
        ;


	/* Stupid commas!!  they make these rules to parse lists
           a little more awkard.
         */
arg_list : '[' arg_list_values ']'
	{
	  $$=$2;
        }
	| '[' ']' /* empty */
        {
	  if (!($$ = al_list_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't alloc new ARG_LIST (emtpy list)");
	  } 
        }
         ;

arg_list_values : arg_list_values  ',' AL_STRING
        {
	  if (al_list_add_string($1,$3) != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add string %s to ARG_LIST", $1);
	  }
	  free ($3);
	  $$=$1;
        }
	| AL_STRING
        {
	  if (!($$ = al_list_new())) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't alloc new ARG_LIST (Adding %s)", $1);
	  } 
	  if (al_list_add_string($$,$1) != AL_E_OK) {
	    /* FIXME: free up stuff to this point */
	    yyerror("Couldn't add string %s to ARG_LIST", $1);
	  }
	  free ($1);
	}
	;

%%

#define MAX_ERRORSTR_LEN	1024

static void yyerror(char *string, ...)
{
	char error[MAX_ERRORSTR_LEN + 1];
	va_list va;

	va_start(va, string);
	g_vsnprintf(error, MAX_ERRORSTR_LEN, string, va);
	va_end(va);

	/* FIXME: print out the line that is in error (possible ?) */
	err_warn("Error in configuration line %d: %s", yy_al_lineno, error);
}

