/* template.c - skel-file for dents-tests

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

/* It might be a good idea to use the glib-functions instead of (f)printf,
   but since it's just a 'good-style' problem I won't rewrite it. */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

#include <stdio.h>		/* printf, fprintf */

#ifdef HAVE_UNISTD_H
#include <unistd.h>		/* dup */
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_GETOPT_H
#include <getopt.h>		/* getopt_long */
#endif /* HAVE_GETOPT_H */

/* Add dents headers here */
#include "../include/common.h"

/* Use this for human-readable regression-test data */
FILE *regout;

/* Use this for boring or technical regression-test data */
FILE *spamout;

/* Call me to initialize streams */
void init_teststreams (int argc, char *argv[]);

/* Example usage */
int
main (int argc, char *argv[])
{
  init_teststreams (argc, argv);

  /* --verbose [nonterministic things] */
  printf ("The checks needed 1.203 seconds\n");

  /* --regress || --verbose [debug-output] */
  fprintf (regout, "Data: YT-1300\n");

  /* --regress [really boring data / data not usefull for humans] */
  fprintf (spamout, "0101010101010101010101\n");

  /* Errors */
  if (1.999 != 2) {
    fprintf (stderr, "Arithemtic assertion failed. Pentium required.\n");
    exit (1);			/* Allways exit(1) after any detected error */
  }
}

#ifdef HAVE_GETOPT_H
/* This will be used by getopt_long() */
static const struct option testopts[] = {
  {"all", 0, NULL, 'a'},
  {"help", 0, NULL, 'h'},
  {"regress", 0, NULL, 'r'},
  {"verbose", 0, NULL, 'v'},
  {NULL}
};
#endif /* HAVE_GETOPT_H */

/* Call me to initialize streams */
void
init_teststreams (int argc, char *argv[])
{
  int verbose = 0;
  int regress = 0;
  int opt;
#ifdef HAVE_GETOPT_H
  while (EOF != (opt = getopt_long (argc, argv, "ahrv", testopts, NULL))) {
#else /* ! HAVE_GETOPT_H */
  while (EOF != (opt = getopt(argc, argv, "ahrv"))) {
#endif /* ! HAVE_GETOPT_H */
    switch (opt) {
    case 'a':
      verbose = 1;
      regress = 1;
      break;
    case '?':
    case 'h':
      printf ("Valid options are: --regress --verbose --all\n\n");
      exit (1);
      break;
    case 'r':
      regress = 1;
      break;
    case 'v':
      verbose = 1;
      break;
    }
  }
  if (verbose || regress) {
    regout = fdopen (dup (fileno (stdout)), "w");
  } else {
    regout = fopen ("/dev/null", "w");
  }
  if (regress) {
    spamout = fdopen (dup (fileno (stdout)), "w");
  } else {
    spamout = fopen ("/dev/null", "w");
  }
  if (!verbose) {
    freopen ("/dev/null", "w", stdout);
  }
}
