/* test_al_parse.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/*
 * test al_hnd_write() al_hnd_read() functions
 */
#include <stdio.h>

#include "arg_list_api.h"

int verbose = 0;

char *good_test_strings[] = {
  "TEST_DRIVER = { 'name':'value'\n\
} ",

  "TEST_DRIVER = {\n\
  'test-list':['apples', 'oranges', 'bananas']} ",
  "TEST_DRIVER = {\n\
  'name-num-1':12345,\n\
  'name-string-1':'value', 'name-list-1':['1', '2', '3', '4']}\n\
",
  "TEST_DRIVER = { \n\
  'name-string-1':'value',\n\
  'name-num-1':12345,\n\
  'name-list-1':['1', '2', '3', '4'], 'name-list-3':['1'], 'name-list-2':[]\n\
}\n\
", "\n\
#\n\
# These are comments, by the way\n\
#\n\
  TEST_DRIVER =\n\
{\n\
'name-string-1': 'value', 'name-num-1': 12345, 'name-list-2':[],\n\
'name-list-1': ['1', '2', '3', '4'], 'name-list-3':['1'],\n\
    // this is a comment as well.\n\
'name-list-4':['1', 'supercalifraglisisticexpialidociious'],\n\
'name-hash-1':{\n\
  }\n\
, 'name-hash-2':{\n\
'name-string-1':'value'}\n\
, 'name-hash-3':{\n\
'name-string-1': 'value', 'name-num-1': 12345, 'name-string-2':'value2'}\n\
}\n\
", " TEST_DRIVER = {\n\
}\n\
", " TEST_DRIVER = {\n\
'name-number-1':123456}\n\
", " TEST_DRIVER = {\n\
'name-hnd-1':{\n\
'name-list-1':['']}\n\
}\n\
", "\n\
#\n\
# Here is an example DENTS config file in the format discussed on the\n\
# mailing list week of July 18\n\
#\n\
  DENTS = {\n\
'options':{\n\
# 'forwarders' : ['207.69.188.186']\n\
  }\n\
, 'zone':{\n\
  'mydomain.com.':\n\
    {\n\
    'module':'mod_stddb',\n\
# the default\n\
  'type': 'master', 'file':'db.mydomain'}\n\
  , 'sventech.com':{\n\
# mod_stddb is the default\n\
    'type': 'slave', 'file': 'ca.sventech', 'masters':['205.252.89.20']\n\
    }\n\
  , '127.IN-ADDR.ARPA':\n\
    {\n\
  'type': 'master', 'file':'db.127'}\n\
  , 'frl.test.':\n\
    {\n\
    'module': 'mod_frl', 'prefix':'user-'\n\
# make this configurable?\n\
    }\n\
  , '159.69.207.in-addr.arpa.':{\n\
  'zone-name': 'frl.test', 'prefix':'user-'}\n\
  , 'zundel.com.':{\n\
    'module': 'mod_mysql_eza', 'db_name': 'dents_test', 'user':'dents',\n\
  'password':'!dented!'}\n\
  }\n\
, 'module':{\n\
  'mod_stddb':{\n\
  'directory':'./db'}\n\
  , 'mod_mysql_eza':{\n\
    }\n\
  }\n\
}\n\
", NULL
};
char *bad_test_strings[] = {
  "'DENTS_TEST' = { foo }",
  "'DENTS_TEST' = {,}",
  "'DENTS_TEST' = { ['1','2','3'] }",
  "'DENTS_TEST' = { 'name1' : 'name2' : 'name3 ' }", NULL
};

int
test_file_input (void)
{
  FILE *infile;
  ARG_HND test_hnd;
  char ret_name[80];
  char **filename;
  char *good_filenames[] = {
    "./test_al_parse_input/test1.in",
    "./test_al_parse_input/test2.in",
    "./test_al_parse_input/test3.in",
    "./test_al_parse_input/test4.in",
    "./test_al_parse_input/test5.in",
    "./test_al_parse_input/test6.in",
    "./test_al_parse_input/test7.in",
    "./test_al_parse_input/test8.in",
    "./test_al_parse_input/test9.in",
    NULL
  };

  char *bad_filenames[] = {
    "./test_al_parse_input/test1.err",
    "./test_al_parse_input/test2.err",
    "./test_al_parse_input/test3.err",
    "./test_al_parse_input/test4.err",
    NULL
  };

  for (filename = good_filenames; filename && *filename; filename++) {

    infile = fopen (*filename, "r");
    if (!infile) {
      fprintf (stderr, "Inputfile not found (%s). NOT TESTED\n", *filename);
      fflush (stderr);
      return 1;
    }

    if ((test_hnd = al_hnd_read (infile, ret_name, sizeof (ret_name))) !=
	NULL) {
      if (verbose) {
	al_hnd_write (test_hnd, stdout, ret_name);
	fflush (stdout);
      }
    } else {
      fprintf (stderr, "Error parsing %s\n", *filename);
      fflush (stderr);
      return 1;
    }
    fclose (infile);
  }

  for (filename = bad_filenames; filename && *filename; filename++) {

    infile = fopen (*filename, "r");
    if (!infile) {
      fprintf (stderr, "Inputfile not found (%s). NOT TESTED\n", *filename);
      fflush (stderr);
      return 1;
    }
#ifdef DENTS_DEBUG
    fprintf (stderr, "Expect a parse error for %s\n", *filename);
    fflush (stderr);
#endif

    if ((test_hnd = al_hnd_read (infile, ret_name, sizeof (ret_name))) !=
	NULL) {
      if (verbose) {
	al_hnd_write (test_hnd, stdout, ret_name);
	fflush (stdout);
      }
      fprintf (stderr, "ERROR: Parse should have failed\n");
      fflush (stderr);
      return 1;
    }

    fclose (infile);
  }

  return 0;
}				/* end test_file_input () */

int
test_string_input (void)
{
  char **test_string;
  ARG_HND test_hnd;
  char ret_name[80];

  for (test_string = good_test_strings;
       test_string && *test_string; test_string++) {

    if (
	(test_hnd =
	 al_hnd_read_string (*test_string, ret_name,
			     sizeof (ret_name))) != NULL) {
      if (verbose) {
	al_hnd_write (test_hnd, stdout, ret_name);
	fflush (stdout);
      }
    } else {
      fprintf (stderr, "Error parsing %s\n", *test_string);
      fflush (stderr);
      exit (1);
    }
  }

  for (test_string = bad_test_strings;
       test_string && *test_string; test_string++) {

#ifdef DENTS_DEBUG
    fprintf (stderr, "Expect a parse error\n");
    fflush (stderr);
#endif

    if (
	(test_hnd =
	 al_hnd_read_string (*test_string, ret_name,
			     sizeof (ret_name))) != NULL) {
      if (verbose) {
	al_hnd_write (test_hnd, stdout, ret_name);
	fflush (stdout);
      }
      fprintf (stderr, "Should have gotten an error parsing %s\n",
	       *test_string);
      fflush (stderr);
      exit (1);
    }
  }

  return 0;

}				/* end test_string_input() */

int
main (int argc, char *argv[])
{

  if (argc > 1)
    verbose++;

  /* Reading from test files doesn't work with some of the configure scripts 
   */
#if TEST_FILE_INPUT
  if (test_file_input ()) {
    return 1;
  }
#endif

  if (test_string_input ()) {
    return 1;
  }

  /* TODO: write the output into a FILE * using arg_hnd_write(), 
     then rerad it back in using arg_hnd_read() and then do a comparison
     to make sure all the data is there!
   */

  return 0;
}				/* end main() */
