/* test_arg_list.c - XXCOMMENTXX

   Dents - a threaded modular dns-server.
   Copyright (C) 1999  Dents Team  (http://www.dents.org)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version. This program is distributed in the
   hope that it will be useful, but WITHOUT ANY WARRANTY; without even
   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Add system headers here */
#include <glib.h>

/* Add dents headers here */
#include "../include/common.h"

/* test_arg_list.c - test driver for 'arg_list_api.c' */

#include <stdio.h>
#include "arg_list_api.h"

int
main (int argc, char *argv[])
{
  ARG_HND test_hnd = NULL;
  ARG_HND test_hnd2 = NULL;
  ARG_LIST test_list = NULL;
  ARG_ENTRY test_entry = NULL;
  float my_float_val = 3.14;
  const int verbose = (int) (argc > 1);

  test_hnd = al_hnd_new ();

  if (al_hnd_add_string (test_hnd, "test-string", "test-string-value") !=
      AL_E_OK) {
    fprintf (stderr, "Error inserting string.\n");
    exit (1);
  }

  if (al_hnd_add_int (test_hnd, "test-int", 666) != AL_E_OK) {
    fprintf (stderr, "Error inserting string.\n");
    exit (1);
  }

  if (al_hnd_add_void (test_hnd, "test-void", (void *) &my_float_val) !=
      AL_E_OK) {
    fprintf (stderr, "Error inserting string.\n");
    exit (1);
  }

  test_list = al_list_new ();
  al_list_add_string (test_list, "apples");
  al_list_add_string (test_list, "oranges");
  al_list_add_string (test_list, "bananas");

  if (al_hnd_add_list (test_hnd, "test-list", test_list) != AL_E_OK) {
    fprintf (stderr, "Error inserting  ARG_LIST.\n");
    exit (1);
  }

  /* allocate a new handle structure */
  test_hnd2 = al_hnd_new ();
  al_hnd_add_string (test_hnd2, "test2", "test2-value");

  if (al_hnd_add_hnd (test_hnd, "test-hnd", test_hnd2) != AL_E_OK) {
    fprintf (stderr, "Error inserting ARG_HND.\n");
    exit (1);
  }

  /* get the list of names */
  if (NULL == (test_list = al_hnd_get_names (test_hnd))) {
    fprintf (stderr, "Retrieving list of names returned NULL\n");
    exit (1);
  }

  for (test_entry = al_list_first (test_list);
       test_entry; test_entry = al_list_next (test_entry)) {
    char *name = al_entry_get_string (test_entry);
    if (verbose) {
      printf ("%s ", name);
    }
  }
  if (verbose) {
    printf ("\n");
  }

  /* test the al_list_string_exists() function */
  if (al_list_string_exists (test_list, "test-int") != 1) {
    fprintf (stderr, "should have found 'test-int' in the list\n");
    exit (1);
  }

  if (al_list_string_exists (test_list, "bogus") == 1) {
    fprintf (stderr, "should NOT have found 'bogus' in the list\n");
    exit (1);
  }

  /* get the integer arg */
  {
    int int_val = al_hnd_find_int (test_hnd, "test-int");
    if (int_val != 666) {
      fprintf (stderr, "Error fetching int.\n");
      exit (1);
    }
  }

  /* get the string arg */
  {
    char *ret_val = al_hnd_find_string (test_hnd, "test-string");
    if (!ret_val || (!g_str_equal (ret_val, "test-string-value"))) {
      fprintf (stderr, "Error fetching string.\n");
      exit (1);
    }
  }

  /* get the void * arg */
  {
    void *ret_val = al_hnd_find_void (test_hnd, "test-void");
    if (!ret_val) {
      fprintf (stderr, "Error fetching void.\n");
      exit (1);
    }
    if (verbose) {
      printf ("fetched ptr to float %f\n", *(float *) ret_val);
    }
  }

  /* get the ARG_LIST arg */
  {
    ARG_LIST ret_val = al_hnd_find_list (test_hnd, "test-list");
    ARG_ENTRY entry;
    if (!ret_val) {
      fprintf (stderr, "Error fetching ARG_LIST.\n");
      exit (1);
    }
    for (entry = al_list_first (ret_val); entry; entry = al_list_next (entry)) {
      if (verbose) {
	printf ("%s ", al_entry_get_string (entry));
      }
    }
    if (verbose) {
      printf ("\n");
    }
  }

  /* get the ARG_HND arg */
  {
    ARG_HND ret_val = al_hnd_find_hnd (test_hnd, "test-hnd");
    char *test2_ret_val;

    if (!ret_val) {
      fprintf (stderr, "Error fetching ARG_HND.\n");
      exit (1);
    }
    if (ret_val != test_hnd2) {
      fprintf (stderr,
	       "Error fetching ARG_HND - I should have found test2_hnd!.\n");
      exit (1);
    }

    if ((test2_ret_val = al_hnd_find_string (ret_val, "test2")) == NULL) {
      fprintf (stderr, "Error fetching from returned ARG_HND.\n");
      exit (1);
    }

    if (!g_str_equal (test2_ret_val, "test2-value")) {
      fprintf (stderr, "Wrong string returned from returned ARG_HND.\n");
      exit (1);
    }

    if (verbose) {
      printf ("returned from test2_hnd = %s\n", test2_ret_val);
    }

  }

  /* get the int arg with the string arg function (should fail) */
  {
    char *ret_val = al_hnd_find_string (test_hnd, "test-int");
    if (ret_val != NULL) {
      fprintf (stderr,
	       "Tried to fetch an int value as a string and it worked??!?\n");
      exit (1);
    }
  }

  /* get the int arg with the void arg function (should fail) */
  {
    void *ret_val = al_hnd_find_void (test_hnd, "test-int");
    if (ret_val != NULL) {
      fprintf (stderr,
	       "Tried to fetch an int value as a void * and it worked??!?\n");
      exit (1);
    }
  }

  /* get the string arg with the int arg function (should fail) */
  {
    int ret_val = al_hnd_find_int (test_hnd, "test-string");
    if (ret_val != -1) {
      fprintf (stderr,
	       "Tried to fetch a string value as a int and it worked??!?\n");
      exit (1);
    }
  }

  /* get the int arg with the list arg function (should fail) */
  {
    ARG_LIST ret_val = al_hnd_find_list (test_hnd, "test-int");
    if (ret_val != NULL) {
      fprintf (stderr,
	       "Tried to fetch an int value as a list and it worked??!?\n");
      exit (1);
    }
  }

  /* get the int arg with the ARG_HND function (should fail) */
  {
    ARG_HND ret_val = al_hnd_find_hnd (test_hnd, "test-int");
    if (ret_val != NULL) {
      fprintf (stderr,
	       "Tried to fetch an int value as a hnd and it worked??!?\n");
      exit (1);
    }
  }

  al_hnd_delete (&test_hnd);

  if (verbose) {
    printf ("All tests passed\n");
  }

  return 0;
}
